from insights.parsers.snmp import TcpIpStats
from insights.parsers.snmp import TcpIpStatsIPV6
from insights.tests import context_wrap

PROC_SNMP = """
Ip: Forwarding DefaultTTL InReceives InHdrErrors InAddrErrors ForwDatagrams InUnknownProtos InDiscards InDelivers OutRequests OutDiscards OutNoRoutes ReasmTimeout ReasmReqds ReasmOKs ReasmFails FragOKs FragFails FragCreates
Ip: 2 64 2628 0 2 0 0 0 2624 1618 0 0 0 0 0 10 0 0 0
Icmp: InMsgs InErrors InDestUnreachs InTimeExcds InParmProbs InSrcQuenchs InRedirects InEchos InEchoReps InTimestamps InTimestampReps InAddrMasks InAddrMaskReps OutMsgs OutErrors OutDestUnreachs OutTimeExcds OutParmProbs OutSrcQuenchs OutRedirects OutEchos OutEchoReps OutTimestamps OutTimestampReps OutAddrMasks OutAddrMaskReps
Icmp: 0 0 0 0 0 0 0 0 0 0 0 0 0 2 0 2 0 0 0 0 0 0 0 0 0 0
IcmpMsg: InType3 OutType3
IcmpMsg: 34 44
Tcp: RtoAlgorithm RtoMin RtoMax MaxConn ActiveOpens PassiveOpens AttemptFails EstabResets CurrEstab InSegs OutSegs RetransSegs InErrs OutRsts
Tcp: 1 200 120000 -1 25 4 0 0 1 2529 1520 1 0 9
Udp: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors
Udp: 95 0 0 95 1 4
UdpLite: InDatagrams NoPorts InErrors OutDatagrams RcvbufErrors SndbufErrors
UdpLite: 0 10 0 0 0 100
""".strip()

PROC_SNMP_NO = """
""".strip()

PROC_SNMP6 = """
Ip6InReceives                   	757
Ip6InHdrErrors                  	0
Ip6InTooBigErrors               	0
Ip6InNoRoutes                   	0
Ip6InAddrErrors                 	0
Ip6InUnknownProtos              	0
Ip6InTruncatedPkts              	0
Ip6InDiscards                   	0
Ip6InDelivers                   	748
Ip6OutForwDatagrams             	0
Ip6OutRequests                  	713
Ip6OutDiscards                  	0
Ip6OutNoRoutes                  	0
Ip6ReasmTimeout                 	0
Ip6ReasmReqds                   	0
Ip6ReasmOKs                     	0
Ip6ReasmFails                   	0
Ip6FragOKs                      	0
Ip6FragFails                    	0
Ip6FragCreates                  	0
Ip6InMcastPkts                  	99
Ip6OutMcastPkts                 	71
Ip6InOctets                     	579410
Ip6OutOctets                    	1553244
Ip6InMcastOctets                	9224
Ip6OutMcastOctets               	5344
Ip6InBcastOctets                	0
Ip6OutBcastOctets               	0
Ip6InNoECTPkts                  	759
Ip6InECT1Pkts                   	0
Ip6InECT0Pkts                   	0
Ip6InCEPkts                     	0
Icmp6InMsgs                     	94
Icmp6InErrors                   	0
Icmp6OutMsgs                    	41
Icmp6OutErrors                  	0
Icmp6InCsumErrors               	0
Icmp6InDestUnreachs             	0
Icmp6InPktTooBigs               	0
Icmp6InTimeExcds                	0
Icmp6InParmProblems             	0
Icmp6InEchos                    	0
Icmp6InEchoReplies              	0
Icmp6InGroupMembQueries         	28
Icmp6InGroupMembResponses       	0
Icmp6InGroupMembReductions      	0
Icmp6InRouterSolicits           	0
Icmp6InRouterAdvertisements     	62
Icmp6InNeighborSolicits         	3
Icmp6InNeighborAdvertisements   	1
Icmp6InRedirects                	0
Icmp6InMLDv2Reports             	0
Icmp6OutDestUnreachs            	0
Icmp6OutPktTooBigs              	0
Icmp6OutTimeExcds               	0
Icmp6OutParmProblems            	0
Icmp6OutEchos                   	0
Icmp6OutEchoReplies             	0
Icmp6OutGroupMembQueries        	0
Icmp6OutGroupMembResponses      	0
Icmp6OutGroupMembReductions     	0
Icmp6OutRouterSolicits          	1
Icmp6OutRouterAdvertisements    	0
Icmp6OutNeighborSolicits        	3
Icmp6OutNeighborAdvertisements  	3
Icmp6OutRedirects               	0
Icmp6OutMLDv2Reports            	34
Icmp6InType130                  	28
Icmp6InType134                  	62
Icmp6InType135                  	3
Icmp6InType136                  	1
Icmp6OutType133                 	1
Icmp6OutType135                 	3
Icmp6OutType136                 	3
Icmp6OutType143                 	34
Udp6InDatagrams                 	0
Udp6NoPorts                     	0
Udp6InErrors                    	0
Udp6OutDatagrams                	0
Udp6RcvbufErrors                	0
Udp6SndbufErrors                	0
Udp6InCsumErrors                	0
UdpLite6InDatagrams             	0
UdpLite6NoPorts                 	0
UdpLite6InErrors                	0
UdpLite6OutDatagrams            	0
UdpLite6RcvbufErrors            	0
UdpLite6SndbufErrors            	0
UdpLite6InCsumErrors            	0
""".strip()

PROC_SNMP6_ODD = """
Ip6InReceives                   	757
Ip6InHdrErrors                  	0
Icmp6OutMLDv2Reports            	0
Icmp6InType130                  	28
Icmp6InType134                  	62
Ip6InDiscards
""".strip()


def test_snmp():
    stats = TcpIpStats(context_wrap(PROC_SNMP))
    snmp_stats = stats.get("Ip")
    assert snmp_stats
    assert snmp_stats["DefaultTTL"] == 64
    assert snmp_stats["InReceives"] == 2628
    assert snmp_stats["InHdrErrors"] == 0
    assert snmp_stats["InAddrErrors"] == 2
    assert snmp_stats["InDiscards"] == 0
    assert snmp_stats["InDelivers"] == 2624
    assert snmp_stats["ReasmFails"] == 10
    assert snmp_stats["OutRequests"] == 1618

    snmp_stats = stats.get("Tcp")
    assert snmp_stats["RtoMax"] == 120000
    assert snmp_stats["MaxConn"] == -1
    assert snmp_stats["OutSegs"] == 1520
    assert snmp_stats["ActiveOpens"] == 25

    snmp_stats = stats.get("IcmpMsg")
    assert snmp_stats["OutType3"] == 44

    snmp_stats = stats.get("Udp")
    assert snmp_stats["OutDatagrams"] == 95
    assert snmp_stats["RcvbufErrors"] == 1
    assert snmp_stats["NoPorts"] == 0

    stats = TcpIpStats(context_wrap(PROC_SNMP_NO))
    snmp_stats = stats.get("Ip")
    assert snmp_stats is None


def test_snmp6():
    stats = TcpIpStatsIPV6(context_wrap(PROC_SNMP6))
    snmp6_stats_RX = stats.get("Ip6InReceives")
    snmp6_stats_MLD = stats.get("Icmp6OutMLDv2Reports")
    assert snmp6_stats_RX == 757
    assert snmp6_stats_MLD == 34
    stats = TcpIpStatsIPV6(context_wrap(PROC_SNMP6_ODD))
    snmp6_stats_disx = stats.get("Ip6InDiscards")
    snmp6_stats_odd = stats.get("some_unknown")
    assert snmp6_stats_disx is None
    assert snmp6_stats_odd is None



from insights.components.satellite import IsCapsule, IsSatellite, IsSatellite611
from insights.combiners.satellite_version import SatelliteVersion, CapsuleVersion
from insights.parsers.installed_rpms import InstalledRpms
from insights.tests import context_wrap
from insights.core.dr import SkipComponent
from insights.core import ParseException
import pytest


NOT_SATELITE_RPMS = """
ConsoleKit-0.4.1-3.el6.x86_64                               Wed Sep 17 11:34:25 2014
kernel-2.6.32-573.el6.x86_64                                Wed May 18 14:17:17 2016
""".strip()

SUPPORTED_SATELLITE_VERSION_610_INSTALLED_RPMS = """
ConsoleKit-0.4.1-3.el6.x86_64                               Wed Sep 17 11:34:25 2014
kernel-2.6.32-573.el6.x86_64                                Wed May 18 14:17:17 2016
satellite-6.10.0.11-1.el7sat.noarch                         Wed May 18 14:16:25 2016
""".strip()

SUPPORTED_SATELLITE_VERSION_611_INSTALLED_RPMS = """
ConsoleKit-0.4.1-3.el6.x86_64                               Wed Sep 17 11:34:25 2014
kernel-2.6.32-573.el6.x86_64                                Wed May 18 14:17:17 2016
satellite-6.11.0-1.el7sat.noarch                            Wed May 18 14:16:25 2016
""".strip()

SUPPORTED_SATELLITE_CAPSULE_VERSION_69_INSTALLED_RPMS = """
satellite-capsule-6.9.0-1.el7sat.noarch                     Thu Jan  9 12:26:31 2020
satellite-installer-6.9.0.4-1.el7sat.noarch                 Thu Jan  9 12:26:26 2020
""".strip()

SUPPORTED_SATELLITE_CAPSULE_VERSION_610_INSTALLED_RPMS = """
satellite-capsule-6.10.3-1.el7sat.noarch                     Thu Jan  9 12:26:31 2020
satellite-installer-6.10.3-1.el7sat.noarch                   Thu Jan  9 12:26:26 2020
""".strip()


# IsSatellite test
def test_is_satellite():
    rpms = InstalledRpms(context_wrap(SUPPORTED_SATELLITE_VERSION_610_INSTALLED_RPMS))
    sat_ver = SatelliteVersion(rpms, None)
    result = IsSatellite(sat_ver)
    assert result is not None
    result_6 = IsSatellite(sat_ver, 6)
    assert result_6 is not None
    result_610 = IsSatellite(sat_ver, 6, 10)
    assert result_610 is not None


def test_is_satellite_except():
    rpms = InstalledRpms(context_wrap(NOT_SATELITE_RPMS))
    with pytest.raises(SkipComponent):
        SatelliteVersion(rpms, None)

    rpms = InstalledRpms(context_wrap(SUPPORTED_SATELLITE_VERSION_610_INSTALLED_RPMS))
    sat_ver = SatelliteVersion(rpms, None)
    with pytest.raises(ParseException):
        IsSatellite(sat_ver, None, 11)
    with pytest.raises(SkipComponent):
        IsSatellite(sat_ver, 5)
    with pytest.raises(SkipComponent):
        IsSatellite(sat_ver, 6, 11)


# IsCapsule test
def test_is_satellite_capsule():
    rpms = InstalledRpms(context_wrap(SUPPORTED_SATELLITE_CAPSULE_VERSION_69_INSTALLED_RPMS))
    cap_ver = CapsuleVersion(rpms, None)
    result = IsCapsule(cap_ver)
    assert result is not None
    result_6 = IsCapsule(cap_ver, 6)
    assert result_6 is not None
    result_69 = IsCapsule(cap_ver, 6, 9)
    assert result_69 is not None


def test_is_capsule_except():
    rpms = InstalledRpms(context_wrap(NOT_SATELITE_RPMS))
    with pytest.raises(SkipComponent):
        CapsuleVersion(rpms, None)
    rpms = InstalledRpms(context_wrap(SUPPORTED_SATELLITE_CAPSULE_VERSION_610_INSTALLED_RPMS))
    cap_ver = CapsuleVersion(rpms, None)
    with pytest.raises(ParseException):
        IsCapsule(cap_ver, None, 11)
    with pytest.raises(SkipComponent):
        IsCapsule(cap_ver, 5)
    with pytest.raises(SkipComponent):
        IsCapsule(cap_ver, 6, 11)


# Satellite611 Tests
def test_is_satellite611():
    rpms = InstalledRpms(context_wrap(SUPPORTED_SATELLITE_VERSION_611_INSTALLED_RPMS))
    sat_ver = SatelliteVersion(rpms, None)
    result = IsSatellite611(sat_ver)
    assert isinstance(result, IsSatellite611)


def test_not_satellite611():
    rpms = InstalledRpms(context_wrap(SUPPORTED_SATELLITE_VERSION_610_INSTALLED_RPMS))
    sat_ver = SatelliteVersion(rpms, None)
    with pytest.raises(SkipComponent) as e:
        IsSatellite611(sat_ver)
    assert "Not a Satellite 6.11" in str(e)



import doctest
import pytest
from insights.parsers import firewall_cmd
from insights.parsers.firewall_cmd import FirewallCmdListALLZones
from insights.tests import context_wrap
from insights.parsers import ParseException
from insights.core.plugins import ContentException

FIREWALL_LIST_ZONES_1 = """
FirewallD is not running
""".strip()

FIREWALL_LIST_ZONES_2 = """
-bash: firewall-cmd: command not found
""".strip()

FIREWALL_LIST_ZONES_3 = """
block
  target: %%REJECT%%
  icmp-block-inversion: no
  interfaces:
  sources:
  services:
  ports:
  protocols:
  masquerade: no
  forward-ports:
  source-ports:
  icmp-blocks:
  rich rules:


dmz
  target: default
  icmp-block-inversion: no
  interfaces:
  sources:
  services: ssh
  ports:
  protocols:
  masquerade: no
  forward-ports:
  source-ports:
  icmp-blocks:
  rich rules:


public (active, default)
  target: default
  icmp-block-inversion: no
  interfaces: eno1
  sources:
  services: dhcpv6-client ssh
  ports:
  protocols:
  masquerade: no
  forward-ports: port=80:proto=tcp:toport=12345:toaddr=
        port=81:proto=tcp:toport=1234:toaddr=
        port=83:proto=tcp:toport=456:toaddr=10.72.47.45
  source-ports:
  icmp-blocks:
  rich rules:
        rule family="ipv4" source address="10.0.0.0/24" destination address="192.168.0.10/32" port port="8080-8090" protocol="tcp" accept
        rule family="ipv4" source address="10.0.0.0/24" destination address="192.168.0.10/32" port port="443" protocol="tcp" reject
        rule family="ipv4" source address="192.168.0.10/24" reject
        rule family="ipv6" source address="1:2:3:4:6::" forward-port port="4011" protocol="tcp" to-port="4012" to-addr="1::2:3:4:7"


trusted
  target: ACCEPT
  icmp-block-inversion: yes
  interfaces:
  sources:
  services:
  ports:
  protocols:
  masquerade: no
  forward-ports:
  source-ports:
  icmp-blocks:
  rich rules:
""".strip()

FIREWALL_LIST_ZONES_4 = """
public (active)
    target: default
    icmp-block-inversion: no
    interfaces: eno1
    sources:
    services: dhcpv6-client ssh
    ports:
    protocols:
    masquerade: no
    forward-ports:
    source-ports
    icmp-blocks
    rich rules
""".strip()


def test_docs():
    env = {
        'zones': FirewallCmdListALLZones(context_wrap(FIREWALL_LIST_ZONES_3))
    }
    failed, total = doctest.testmod(firewall_cmd, globs=env)
    assert failed == 0


def test_empty_content():
    with pytest.raises(ContentException):
        FirewallCmdListALLZones(context_wrap(FIREWALL_LIST_ZONES_1))
    with pytest.raises(ContentException):
        FirewallCmdListALLZones(context_wrap(FIREWALL_LIST_ZONES_2))
    with pytest.raises(ParseException):
        FirewallCmdListALLZones(context_wrap(FIREWALL_LIST_ZONES_4))


def test_firewall_info():
    zones = FirewallCmdListALLZones(context_wrap(FIREWALL_LIST_ZONES_3))
    assert 'trusted' not in zones.active_zones
    assert zones.zones['public']['services'] == ['dhcpv6-client ssh']
    assert zones.zones['public']['icmp-block-inversion'] == ['no']
    assert zones.zones['trusted']['services'] == []
    assert zones.zones['trusted']['icmp-block-inversion'] == ['yes']
    zone_info = ['target', 'icmp-block-inversion', 'interfaces', 'sources', 'services',
                         'ports', 'protocols', 'masquerade', 'forward-ports', 'source-ports',
                         'icmp-blocks', 'rich rules']
    assert all(key in zones.zones['public'] for key in zone_info)
    assert 'port=80:proto=tcp:toport=12345:toaddr=' in zones.zones['public']['forward-ports']
    assert 'port=83:proto=tcp:toport=456:toaddr=10.72.47.45' in zones.zones['public']['forward-ports']
    assert len(zones.zones['public']['forward-ports']) == 3
    assert len(zones.zones['public']['rich rules']) == 4
    assert 'active' in zones.zones['public']['_attributes']
    assert 'default' in zones.zones['public']['_attributes']
    assert 'rule family="ipv4" source address="10.0.0.0/24" destination address="192.168.0.10/32" port port="8080-8090" protocol="tcp" accept' in zones.zones['public']['rich rules']
    assert 'rule family="ipv4" source address="192.168.0.10/24" reject' in zones.zones['public']['rich rules']



from insights.combiners.virt_who_conf import AllVirtWhoConf
from insights.parsers.virt_who_conf import VirtWhoConf
from insights.parsers.sysconfig import VirtWhoSysconfig
from insights.tests import context_wrap

VWHO_D_CONF_ESX = """
## This is a template for virt-who configuration files. Please see
## virt-who-config(5) manual page for detailed information.
##
## virt-who checks all files in /etc/virt-who.d/ if they're valid ini-like
## files and uses them as configuration. Each file might contain more configs.
##
## You can uncomment and fill following template or create new file with
## similar content.

## For complete list of options, see virt-who-config(5) manual page.

## Terse version of the config template:
[esx_1]
type=esx
server=10.0.0.219
#encrypted_password=
owner=Satellite
env=Satellite
"""

VWHO_D_CONF_HYPER = """
## This is a template for virt-who configuration files. Please see
## virt-who-config(5) manual page for detailed information.
##
## virt-who checks all files in /etc/virt-who.d/ if they're valid ini-like
## files and uses them as configuration. Each file might contain more configs.
##
## You can uncomment and fill following template or create new file with
## similar content.

## For complete list of options, see virt-who-config(5) manual page.

## Terse version of the config template:
[hyperv_1]
type=hyperv
server=10.0.0.209
#encrypted_password=
owner=Satellite
env=Satellite
"""

VWHO_CONF = """
## This is a template for virt-who global configuration files. Please see
## virt-who-config(5) manual page for detailed information.
##
## virt-who checks /etc/virt-who.conf for sections 'global' and 'defaults'.
## The sections and their values are explained below.
## NOTE: These sections retain their special meaning and function only when present in /etc/virt-who.conf
##
## You can uncomment and fill following template or create new file with
## similar content.

#Terse version of the general config template:
[global]

interval=3600
debug=False
oneshot=False
#log_per_config=False
#log_dir=
#log_file=
#configs=

[defaults]
owner=Satellite
env=Satellite
"""

SYS_VIRTWHO = """
VIRTWHO_BACKGROUND=0
VIRTWHO_ONE_SHOT=1
VIRTWHO_INTERVAL=1000
VIRTWHO_SATELLITE6=1
""".strip()

SYS_VIRTWHO_SAT_LEGACY = """
VIRTWHO_BACKGROUND=1
VIRTWHO_SATELLITE=1
""".strip()

SYS_VIRTWHO_SAM = """
VIRTWHO_SAM=1
""".strip()

SYS_VIRTWHO_CP = """
VIRTWHO_SAM=0
""".strip()


def test_virt_who_conf_1():
    vw_sysconf = VirtWhoSysconfig(context_wrap(SYS_VIRTWHO))
    vwho_conf = VirtWhoConf(context_wrap(VWHO_CONF))
    vwhod_conf1 = VirtWhoConf(context_wrap(VWHO_D_CONF_HYPER))
    vwhod_conf2 = VirtWhoConf(context_wrap(VWHO_D_CONF_ESX))
    result = AllVirtWhoConf(vw_sysconf, [vwho_conf, vwhod_conf1, vwhod_conf2])
    assert result.background is False
    assert result.oneshot is True
    assert result.interval == 1000
    assert result.sm_type == 'sat6'
    assert sorted(result.hypervisor_types) == sorted(['esx', 'hyperv'])

    expected = [{'name': 'esx_1', 'server': '10.0.0.219', 'env': 'Satellite',
                 'owner': 'Satellite', 'type': 'esx'},
                {'name': 'hyperv_1', 'server': '10.0.0.209', 'env': 'Satellite',
                 'owner': 'Satellite', 'type': 'hyperv'}]
    for d in result.hypervisors:
        assert d in expected


def test_virt_who_conf_2():
    vw_sysconf = VirtWhoSysconfig(context_wrap(SYS_VIRTWHO_SAT_LEGACY))
    vwho_conf = VirtWhoConf(context_wrap(VWHO_CONF))
    result = AllVirtWhoConf(vw_sysconf, [vwho_conf])
    assert result.background is True
    assert result.oneshot is False
    assert result.interval == 3600
    assert result.sm_type == 'sat5'


def test_virt_who_conf_3():
    vw_sysconf = VirtWhoSysconfig(context_wrap(SYS_VIRTWHO_SAM))
    vwho_conf = VirtWhoConf(context_wrap(VWHO_CONF))
    result = AllVirtWhoConf(vw_sysconf, [vwho_conf])
    assert result.sm_type == 'sam'


def test_virt_who_conf_4():
    vw_sysconf = VirtWhoSysconfig(context_wrap(SYS_VIRTWHO_CP))
    vwho_conf = VirtWhoConf(context_wrap(VWHO_CONF))
    result = AllVirtWhoConf(vw_sysconf, [vwho_conf])
    assert result.sm_type == 'cp'



from insights.parsers.tags import Tags
from insights.tests.parsers import skip_exception_check
from insights.tests import context_wrap

tags_json_content = """
{"zone": "east", "owner": "test", "exclude": "true", "group": "app-db-01"}
""".strip()


def test_tags_json():
    ctx = context_wrap(tags_json_content)
    ctx.content = tags_json_content
    result = Tags(ctx)
    assert result.data['zone'] == "east"
    assert result.data['owner'] == "test"
    assert result.data['exclude'] == "true"
    assert result.data['group'] == "app-db-01"


def test_tags_json_bytes():
    bytes_content = bytes(str(tags_json_content).encode("utf-8"))
    ctx = context_wrap(bytes_content)
    ctx.content = bytes_content
    result = Tags(ctx)
    assert result.data['zone'] == "east"
    assert result.data['owner'] == "test"
    assert result.data['exclude'] == "true"
    assert result.data['group'] == "app-db-01"


def test_tags_empty():
    assert 'Empty output.' in skip_exception_check(Tags)



from insights.tests import context_wrap
from insights.parsers.dmidecode import DMIDecode
from insights.combiners.cloud_provider import CloudProvider
from insights.components.cloud_provider import IsAWS, IsAzure, IsGCP
from insights.tests.combiners.test_cloud_provider import (
        DMIDECODE_AWS, DMIDECODE_GOOGLE, DMIDECODE_AZURE_ASSET_TAG)


def test_is_aws():
    dmi = DMIDecode(context_wrap(DMIDECODE_AWS))
    cp = CloudProvider(None, dmi, None, None)
    result = IsAWS(cp)
    assert isinstance(result, IsAWS)


def test_is_azure():
    dmi = DMIDecode(context_wrap(DMIDECODE_AZURE_ASSET_TAG))
    cp = CloudProvider(None, dmi, None, None)
    result = IsAzure(cp)
    assert isinstance(result, IsAzure)


def test_is_gcp():
    dmi = DMIDecode(context_wrap(DMIDECODE_GOOGLE))
    cp = CloudProvider(None, dmi, None, None)
    result = IsGCP(cp)
    assert isinstance(result, IsGCP)



from __future__ import print_function
import os
import os.path
import sys
from getpass import getuser

from insights.core.evaluators import Formatter
from insights.formats import FormatterAdapter
from insights import dr, rule, condition, incident, parser
import logging
from logging import handlers


class SysLogFormat(Formatter):
    """
    This class gathers exceptions and logs the content to syslog. It should be used
    as a context manager and given an instance of an
    ``insights.core.dr.Broker``. ``dr.run`` should be called within the context
    using the same broker.

    Args:
        broker (Broker): the broker to watch and provide a summary about.
    """

    def __init__(self, broker, stream=None, archive=None):
        self.broker = broker
        self.exceptions = []
        self.rules = []
        self.user = getuser()
        self.pid = str(os.getpid())
        self.stream = stream
        self.archive = os.path.realpath(archive) if archive else None

        if not self.stream:
            self.logger = logging.getLogger('sysLogLogger')
            self.logger.propagate = False
            self.logger.setLevel(logging.INFO)

            address = '/dev/log'
            if not os.path.exists(address):
                address = 'localhost', 514

            self.handler = handlers.SysLogHandler(address)
            self.handler.formatter = logging.Formatter('%(message)s')
            self.logger.addHandler(self.handler)

    def logit(self, msg, pid, user, cname, priority=None):
        """Function for formatting content and logging to syslog"""

        if self.stream:
            print(msg, file=self.stream)
        elif priority == logging.WARNING:
            self.logger.warning("{0}[pid:{1}] user:{2}: WARNING - {3}".format(cname, pid, user, msg))
        elif priority == logging.ERROR:
            self.logger.error("{0}[pid:{1}] user:{2}: ERROR - {3}".format(cname, pid, user, msg))
        else:
            self.logger.info("{0}[pid:{1}] user:{2}: INFO - {3}".format(cname, pid, user, msg))

    def log_exceptions(self, c, broker):
        """Gets exceptions to be logged and sends to logit function to be logged to syslog"""

        if c in broker.exceptions:
            ex = broker.exceptions.get(c)
            ex = "Exception in {0} - {1}".format(dr.get_name(c), str(ex))
            self.logit(ex, self.pid, self.user, "insights-run", logging.ERROR)

    def show_tracebacks(self):
        """ Show tracebacks """
        if self.broker.tracebacks:
            for tb in self.broker.tracebacks.values():
                # tb = "Traceback {0}".format(str(tb))
                self.logit(str(tb), self.pid, self.user, "insights-run", logging.ERROR)

    def log_rule_info(self):
        """Collects rule information and send to logit function to log to syslog"""

        for c in sorted(self.broker.get_by_type(rule), key=dr.get_name):
            v = self.broker[c]
            _type = v.get("type")
            if _type:
                if _type != "skip":
                    msg = "Running {0} ".format(dr.get_name(c))
                    self.logit(msg, self.pid, self.user, "insights-run", logging.INFO)
                else:
                    msg = "Rule skipped {0} ".format(dr.get_name(c))
                    self.logit(msg, self.pid, self.user, "insights-run", logging.WARNING)

    def preprocess(self):

        cmd = "Command Line - %s" % " ".join(sys.argv)
        self.logit(cmd, self.pid, self.user, "insights-run", logging.INFO)

        archive = "Real Archive Path - %s" % self.archive
        self.logit(archive, self.pid, self.user, "insights-run", logging.INFO)

        self.broker.add_observer(self.log_exceptions, rule)
        self.broker.add_observer(self.log_exceptions, condition)
        self.broker.add_observer(self.log_exceptions, incident)
        self.broker.add_observer(self.log_exceptions, parser)

    def postprocess(self):

        self.log_rule_info()
        self.show_tracebacks()


class SysLogFormatterAdapter(FormatterAdapter):
    """Logs Rules run, exceptions and tracebacks to syslog."""

    def __init__(self, args):
        self.formatter = None
        self.archive = args.archive

    def preprocess(self, broker):
        self.formatter = SysLogFormat(broker, archive=self.archive)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()



import doctest
from insights.tests import context_wrap
from insights.parsers import swift_conf

proxy_server_conf = """
[DEFAULT]
bind_port = 8080
bind_ip = 172.20.15.20
workers = 0
user = swift
log_name = proxy-server
log_facility = LOG_LOCAL1
log_level = INFO
log_headers = False
log_address = /dev/log
[pipeline:main]
pipeline = catch_errors healthcheck proxy-logging cache ratelimit bulk tempurl formpost authtoken keystone staticweb versioned_writes ceilometer proxy-logging proxy-server
[app:proxy-server]
use = egg:swift#proxy
set log_name = proxy-server
set log_facility = LOG_LOCAL1
set log_level = INFO
set log_address = /dev/log
log_handoffs = true
allow_account_management = true
account_autocreate = true
node_timeout = 60
[filter:catch_errors]
use = egg:swift#catch_errors
[filter:bulk]
use = egg:swift#bulk
max_containers_per_extraction = 10000
max_failed_extractions = 1000
max_deletes_per_request = 10000
yield_frequency = 60
[filter:tempurl]
use = egg:swift#tempurl
[filter:formpost]
use = egg:swift#formpost
[filter:authtoken]
log_name = swift
signing_dir = /var/cache/swift
paste.filter_factory = keystonemiddleware.auth_token:filter_factory
auth_uri = http://172.20.3.28:5000/v2.0
auth_url = http://172.20.4.106:35357
auth_plugin = password
project_domain_id = default
user_domain_id = default
project_name = service
username = swift
delay_auth_decision = 1
cache = swift.cache
include_service_catalog = false
[filter:keystone]
use = egg:swift#keystoneauth
operator_roles = admin, swiftoperator, ResellerAdmin
reseller_prefix = AUTH_
[filter:staticweb]
use = egg:swift#staticweb
url_base = https://10.75.13.138:13808
[filter:versioned_writes]
use = egg:swift#versioned_writes
allow_versioned_writes = true
[filter:ceilometer]
paste.filter_factory = ceilometermiddleware.swift:filter_factory
url = rabbit://guest:J8F22AqWpXqCJyv3mQKeRQszG@172.20.3.20:5672,guest:J8F22AqWpXqCJyv3mQKeRQszG@172.20.3.31:5672,guest:J8F22AqWpXqCJyv3mQKeRQszG@172.20.3.22:5672//
url_test =
"""

object_expirer = """
[DEFAULT]
[object-expirer]
# auto_create_account_prefix = .
auto_create_account_prefix = .
process=0
concurrency=1
recon_cache_path=/var/cache/swift
interval=300
reclaim_age=604800
report_interval=300
processes=0
expiring_objects_account_name=expiring_objects
[pipeline:main]
pipeline = catch_errors cache proxy-server
[app:proxy-server]
use = egg:swift#proxy
[filter:cache]
use = egg:swift#memcache
memcache_servers = 172.16.64.60:11211
[filter:catch_errors]
use = egg:swift#catch_errors
"""

SWIFT_CONF = """
[swift-hash]
# random unique strings that can never change (DO NOT LOSE)
# Use only printable chars (python -c "import string; print(string.printable)")
swift_hash_path_prefix = changeme
swift_hash_path_suffix = changeme

[storage-policy:0]
name = gold
policy_type = replication
default = yes

[storage-policy:1]
name = silver
policy_type = replication

[storage-policy:2]
name = ec42
policy_type = erasure_coding
ec_type = liberasurecode_rs_vand
ec_num_data_fragments = 4
ec_num_parity_fragments = 2
"""


def test_proxy_server_conf():
    result = swift_conf.SwiftProxyServerConf(context_wrap(proxy_server_conf))
    assert 'filter:ceilometer' in result
    assert 'filter:staticweb' in result
    assert result.items('filter:ceilometer').get('url_test') == ''
    assert result.getint('DEFAULT', 'bind_port') == 8080


def test_object_expirer_conf():
    result = swift_conf.SwiftObjectExpirerConf(context_wrap(object_expirer))
    assert 'filter:cache' in result
    assert 'object-expirer' in result
    assert result.get('filter:cache', 'memcache_servers') == '172.16.64.60:11211'
    assert result.getint('object-expirer', 'report_interval') == 300


def test_swift_conf():
    conf = swift_conf.SwiftConf(context_wrap(SWIFT_CONF))
    assert 'swift-hash' in conf.sections()
    assert conf.has_option('storage-policy:2', 'policy_type') is True
    assert conf.get('storage-policy:2', 'policy_type') == 'erasure_coding'
    assert conf.get('storage-policy:2', 'ec_type') == 'liberasurecode_rs_vand'


def test_swift_conf_documentation():
    failed_count, tests = doctest.testmod(
        swift_conf,
        globs={'swift_conf': swift_conf.SwiftConf(context_wrap(SWIFT_CONF)),
               'object_expirer_conf': swift_conf.SwiftObjectExpirerConf(context_wrap(object_expirer)),
               'proxy_server_conf': swift_conf.SwiftProxyServerConf(context_wrap(proxy_server_conf))}
    )
    assert failed_count == 0



"""
Custom datasources to get a list of modules to check the detailed module info.
"""

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.filters import get_filters
from insights.specs import Specs


@datasource(HostContext)
def kernel_module_filters(broker):
    """
    Return a string of a list of modules from the spec filter,
    separated with space.
    """
    filters = sorted((get_filters(Specs.modinfo_modules)))
    if filters:
        return ' '.join(filters)
    raise SkipComponent



import yaml
import pytest
from mock.mock import Mock

from insights.core import filters
from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs import Specs
from insights.specs.datasources.cloud_init import cloud_cfg, LocalSpecs


CLOUD_CFG = """
users:
  - name: demo
    ssh-authorized-keys:
      - key_one
      - key_two
    passwd: $6$j212wezy$7H/1LT4f9/N3wpgNunhsIqtMj62OKiS3nyNwuizouQc3u7MbYCarYeAHWYPYb2FT.lbioDm2RrkJPb9BZMN1O/

ssh_deletekeys: {value}

network:
  version: 1
  config:
    - type: physical
      name: eth0
      subnets:
        - type: dhcp
        - type: dhcp6
""".strip()

CLOUD_CFG_BAD_INDENT = """
#cloud-config
users:
  - name: demo
    ssh-authorized-keys:
      - key_one
      - key_two
    passwd: $6$j212wezy$7H/1LT4f9/N3wpgNunhsIqtMj62OKiS3nyNwuizouQc3u7MbYCarYeAHWYPYb2FT.lbioDm2RrkJPb9BZMN1O/

ssh_deletekeys: 1

network:
  config: disabled

 system_info:
   default_user:
    name: user2
    plain_text_passwd: 'someP@assword'
    home: /home/user2
""".strip()

CLOUD_CFG_BAD = """
users
  name -demo
  ssh-authorized-keys
      - key_one
      - key_two
""".strip()


CLOUD_CFG_FILTERED = """
network:
  config:
  - name: eth0
    subnets:
    - type: dhcp
    - type: dhcp6
    type: physical
  version: 1
ssh_deletekeys: 1
"""

RELATIVE_PATH = '/etc/cloud/cloud.cfg'


def setup_function(func):
    if Specs.cloud_cfg in filters._CACHE:
        del filters._CACHE[Specs.cloud_cfg]
    if Specs.cloud_cfg in filters.FILTERS:
        del filters.FILTERS[Specs.cloud_cfg]

    if func is test_cloud_cfg:
        filters.add_filter(Specs.cloud_cfg, ['ssh_deletekeys', 'network', 'debug'])
    if func is test_cloud_cfg_no_filter:
        filters.add_filter(Specs.cloud_cfg, [])
    elif func is test_cloud_cfg_bad:
        filters.add_filter(Specs.cloud_cfg, ['not_found'])


def teardown_function(func):
    if func is test_cloud_cfg_bad or func is test_cloud_cfg:
        del filters.FILTERS[Specs.cloud_cfg]


@pytest.mark.parametrize("ssh_deletekeys", [0, 1])
def test_cloud_cfg(ssh_deletekeys):
    cloud_cfg_string = CLOUD_CFG.format(value=ssh_deletekeys)
    cloud_cfg_dict = yaml.safe_load(CLOUD_CFG_FILTERED)
    cloud_cfg_dict["ssh_deletekeys"] = ssh_deletekeys

    cloud_init_file = Mock()
    cloud_init_file.content = cloud_cfg_string.splitlines()
    broker = {LocalSpecs.cloud_cfg_input: cloud_init_file}
    result = cloud_cfg(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=yaml.dump(cloud_cfg_dict), relative_path=RELATIVE_PATH)
    assert result.content == expected.content
    assert result.relative_path == expected.relative_path


def test_cloud_cfg_no_filter():
    cloud_init_file = Mock()
    cloud_init_file.content = CLOUD_CFG.format(value=1).splitlines()
    broker = {LocalSpecs.cloud_cfg_input: cloud_init_file}
    with pytest.raises(SkipComponent) as e:
        cloud_cfg(broker)
    assert 'SkipComponent' in str(e)


def test_cloud_cfg_bad():
    cloud_init_file = Mock()
    cloud_init_file.content = CLOUD_CFG_BAD.splitlines()
    broker = {LocalSpecs.cloud_cfg_input: cloud_init_file}
    with pytest.raises(SkipComponent) as e:
        cloud_cfg(broker)
    assert 'Invalid YAML format' in str(e)

    cloud_init_file.content = CLOUD_CFG_BAD_INDENT.splitlines()
    broker = {LocalSpecs.cloud_cfg_input: cloud_init_file}
    with pytest.raises(SkipComponent) as e:
        cloud_cfg(broker)
    assert 'Unexpected exception' in str(e)



"""
General configuration file for NFS daemons and tools - ``/etc/nfs.conf``
========================================================================

This file contains site-specific configuration for various NFS daemons and
other processes. In particular, this encourages consistent configuration
across different processes.

NFSConf - file ``/etc/nfs.conf``
--------------------------------

"""

from insights.core.plugins import parser
from insights.core import IniConfigFile
from insights.specs import Specs


@parser(Specs.nfs_conf)
class NFSConf(IniConfigFile):
    """
    Class parses the ``/etc/nfs.conf`` file using the ``IniConfigFile`` base
    parser.

    .. note::
        In some RHEL version, both the file ``/etc/nfs.conf`` and file
        ``/etc/sysconfig/nfs`` exist, and take effect at the same time for NFS
        services on the host. And it's possible that there are overlaps between
        the two configuration files.
        A combiner for the two configuration files was considered.
        Since the two files' coverage are different, and it is quite complicate
        to enumerate all the configuration options and combine them properly,
        and also, ``/etc/sysconfig/nfs`` is deprecated in lately RHEL releases.
        We deselect it as a consequence.

    Sample configuration file::

        [general]
        # pipefs-directory=/var/lib/nfs/rpc_pipefs

        [exportfs]
        debug=0

        [gssd]
        use-gss-proxy=1

        [nfsd]
        # debug=0
        vers3=n
        # vers4=y
        # vers4.0=y
        # vers4.1=y
        # vers4.2=y
        rdma=y
        rdma-port=20049

    Examples:
        >>> sorted(nfs_conf.sections())
        ['exportfs', 'general', 'gssd', 'nfsd']
        >>> nfs_conf.get('gssd', 'use-gss-proxy')
        '1'
        >>> nfs_conf.getint('gssd', 'use-gss-proxy')
        1
        >>> nfs_conf.get('nfsd', 'vers3')
        'n'
        >>> nfs_conf.get('nfsd', 'rdma-port')
        '20049'
    """
    pass



import doctest
from insights.parsers import gnocchi
from insights.parsers.gnocchi import GnocchiConf, GnocchiMetricdLog
from insights.tests import context_wrap

from datetime import datetime

GNOCCHI_CONF = """
[DEFAULT]
log_dir = /var/log/gnocchi
[api]
auth_mode = keystone
max_limit = 1000
[archive_policy]
[indexer]
url = mysql+pymysql://gnocchi:exampleabckeystring@192.168.0.1/gnocchi?charset=utf8
[metricd]
workers = 2
[oslo_middleware]
[oslo_policy]
policy_file = /etc/gnocchi/policy.json
[statsd]
resource_id = 5e3fcbe2-7aab-475d-b42c-abcdefgh
user_id = e0ca4711-1128-422c-abd6-abcdefgh
project_id = af0c88e8-90d8-4795-9efe-abcdefgh
archive_policy_name = high
flush_delay = 10
[storage]
driver = file
#this one comment
file_basepath = /var/lib/gnocchi
[keystone_authtoken]
auth_uri=http://192.168.0.1:5000/v3
auth_type=password
auth_version=v3
auth_url=http://192.168.0.1:35357
username=gnocchi
password=yourpassword23432
user_domain_name=Default
project_name=services
project_domain_name=Default
""".strip()

METRICD_LOG = """
2017-04-12 03:10:53.076 14550 INFO gnocchi.cli [-] 0 measurements bundles across 0 metrics wait to be processed.
2017-04-12 03:12:53.078 14550 INFO gnocchi.cli [-] 0 measurements bundles across 0 metrics wait to be processed.
2017-04-12 03:14:53.080 14550 INFO gnocchi.cli [-] 0 measurements bundles across 0 metrics wait to be processed.
2017-04-13 21:06:11.676 114807 ERROR tooz.drivers.redis ToozError: Cannot extend an unlocked lock
"""


def test_gnocchi_conf():
    gnocchi_conf = GnocchiConf(context_wrap(GNOCCHI_CONF))
    assert sorted(gnocchi_conf.sections()) == sorted(['api', 'archive_policy', 'indexer', 'metricd', 'oslo_middleware', 'oslo_policy', 'statsd', 'storage', 'keystone_authtoken'])
    assert "storage" in gnocchi_conf.sections()
    assert gnocchi_conf.has_option('indexer', 'url')
    assert gnocchi_conf.get("indexer", "url") == "mysql+pymysql://gnocchi:exampleabckeystring@192.168.0.1/gnocchi?charset=utf8"
    assert gnocchi_conf.getint("statsd", "flush_delay") == 10


def test_metrics_log():
    log = GnocchiMetricdLog(context_wrap(METRICD_LOG))
    assert len(log.get('INFO')) == 3
    assert 'measurements bundles across 0 metrics wait to be processed' in log
    assert log.get('ERROR') == [{'raw_message': '2017-04-13 21:06:11.676 114807 ERROR tooz.drivers.redis ToozError: Cannot extend an unlocked lock'}]
    assert len(list(log.get_after(datetime(2017, 4, 12, 19, 36, 38)))) == 1
    assert list(log.get_after(datetime(2017, 4, 12, 19, 36, 38))) == [{'raw_message': '2017-04-13 21:06:11.676 114807 ERROR tooz.drivers.redis ToozError: Cannot extend an unlocked lock'}]


def test_doc():
    env = {
            'GnocchiConf': GnocchiConf,
            'conf': GnocchiConf(context_wrap(GNOCCHI_CONF, path='/etc/gnocchi/gnocchi.conf')),
            'GnocchiMetricdLog': GnocchiMetricdLog,
            'log': GnocchiMetricdLog(context_wrap(METRICD_LOG, path='/etc/gnocchi/metricd.log')),
          }
    failed, total = doctest.testmod(gnocchi, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.parsers.cron_jobs import CronFile, CronForeman
from insights.parsers import ParseException, cron_jobs
from insights.tests import context_wrap

CRON_FILE_SAMPLE1 = """
SHELL=/bin/sh
TEST_HOME=/usr/share/test

15 23 * * *     test    /usr/sbin/testaa 2>&1 >> /var/log/test.log
0 7 * * *       test    /usr/sbin/testbb 2>&1 >> /var/log/test.log
"""

CRON_FILE_SAMPLE2 = """
SHELL=/bin/sh
TEST_HOME=/usr/share/test

testinvliadline
15 23 * * *     test    /usr/sbin/testaa 2>&1 >> /var/log/test.log
@daily       test    /usr/sbin/testbb 2>&1 >> /var/log/test.log
30 7 * * *      test    /usr/sbin/testcc 2>&1 >> /var/log/test.log
30 8 * * *      user1    /usr/sbin/someone 2>&1 >> /var/log/user.log
30 9 * * *      user1    /usr/sbin/sometwo 2>&1 >> /var/log/user.log
30 10 * * *     user1    /usr/sbin/somethree 2>&1 >> /var/log/user.log
30 11 * * *      user1    /usr/sbin/somefour 2>&1 >> /var/log/user.log
@reboot     user1    /usr/sbin/somefive
"""

CRON_FILE_SAMPLE3 = """
SHELL=/bin/sh
TEST_HOME=/usr/share/test

testinvliadline
15 23 * * *     test    /usr/sbin/testaa 2>&1 >> /var/log/test.log
@wrongnickname       test    /usr/sbin/testbb 2>&1 >> /var/log/test.log
30 7 * * *      test    /usr/sbin/testcc 2>&1 >> /var/log/test.log
@reboot     test    /usr/sbin/testdd 2>&1 >> /var/log/test.log
"""

CRON_FOREMAN_SAMPLE1 = """
SHELL=/bin/sh
PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

RAILS_ENV=production
FOREMAN_HOME=/usr/share/foreman

# Clean up the session entries in the database
15 23 * * *     foreman    /usr/sbin/foreman-rake db:sessions:clear 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log

# Send out recurring notifications
0 7 * * *       foreman    /usr/sbin/foreman-rake reports:daily 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log
0 5 * * 0       foreman    /usr/sbin/foreman-rake reports:weekly 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log
"""

CRON_FOREMAN_SAMPLE2 = """
SHELL=/bin/sh
PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin
testinvaludline

# Clean up the session entries in the database
15 23 * * *     foreman    /usr/sbin/foreman-rake db:sessions:clear 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log
@daily foreman /usr/sbin/foreman-rake reports:expire 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log
@reboot foreman /usr/sbin/foreman-rake reports:clean
"""

CRON_FOREMAN_SAMPLE3 = """
SHELL=/bin/sh
PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

RAILS_ENV=production
FOREMAN_HOME=/usr/share/foreman
testinvaludline

# Clean up the session entries in the database
15 23 * * *     foreman    /usr/sbin/foreman-rake db:sessions:clear 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log

@tenyearly foreman /usr/sbin/foreman-rake reports:expire 2>&1 | gawk '{ print strftime("[\%Y-\%m-\%d \%H:\%M:\%S]"), $0 }' >>/var/log/foreman/cron.log
"""


def test_doc():
    env = {
        'cron_obj': CronFile(context_wrap(CRON_FILE_SAMPLE1)),
        'foreman_jobs': CronForeman(context_wrap(CRON_FOREMAN_SAMPLE1))
    }
    failed, total = doctest.testmod(cron_jobs, globs=env)
    assert failed == 0


def test_cron_file():
    cron_obj = CronFile(context_wrap(CRON_FILE_SAMPLE2))
    assert len(cron_obj.jobs) == 8
    assert len(cron_obj.invalid_lines) == 1
    assert len(cron_obj.environment) == 2
    user1_jobs = cron_obj.search(username='user1')
    assert len(user1_jobs) == 5
    assert user1_jobs[-1].get('time') == '@reboot'
    assert user1_jobs[-1].get('command') == '/usr/sbin/somefive'
    assert user1_jobs[-1].get('raw') == '@reboot     user1    /usr/sbin/somefive'
    test_jobs = cron_obj.search(command__contains='test')
    assert len(test_jobs) == 3
    assert test_jobs[1].get('minute') == '0'
    assert test_jobs[1].get('hour') == '0'
    assert '/usr/sbin/testbb' in test_jobs[1].get('command')


def test_cron_file_except():
    with pytest.raises(ParseException):
        CronForeman(context_wrap(CRON_FILE_SAMPLE3))


def test_cron_foreman():
    cron_foreman = CronForeman(context_wrap(CRON_FOREMAN_SAMPLE2))
    assert len(cron_foreman.jobs) == 3
    assert len(cron_foreman.invalid_lines) == 1
    assert len(cron_foreman.environment) == 2

    assert 'SHELL' in cron_foreman.environment
    assert cron_foreman.environment.get('SHELL') == '/bin/sh'

    assert 'testinvaludline' in cron_foreman.invalid_lines[0]

    assert 'reports:expire' in cron_foreman.jobs[1].get('command')
    assert cron_foreman.jobs[1]['minute'] == '0'
    assert cron_foreman.jobs[1]['hour'] == '0'

    assert 'time' in cron_foreman.jobs[2]
    assert '@reboot' == cron_foreman.jobs[2].get('time')



"""
Sysctl configuration files
==========================
"""
import re

from insights.core.plugins import combiner
from insights.parsers.sysctl import SysctlConf, SysctlDConfEtc, SysctlDConfUsr


@combiner([SysctlConf, SysctlDConfUsr, SysctlDConfEtc])
class SysctlConfs(dict):
    """
    Combiner for accessing all the sysctl configuration files in one
    structure.

    Sample input::

        # sysctl.conf sample
        #
          kernel.domainname = example.com

        ; this one has a space which will be written to the sysctl!
          kernel.modprobe = /sbin/mod probe

    Attributes:
        search(dict): Returns a dict of any kv pairs where the
        contains the search word.

    Examples:
        >>> type(sysctl_conf)
        <class 'insights.combiners.sysctl_conf.SysctlConfs'>
        >>> sysctl_conf['kernel.domainname']
        'example.com'
        >>> sysctl_conf['kernel.modprobe']
        '/sbin/mod probe'
        >>> sysctl_conf['kernel.sysrq']
        '1'
        >>> "vm.dirty_ratio" in sysctl_conf
        True
        >>> sysctl_conf.search("domainname")
        {'kernel.domainname': 'example.com'}
    """
    def __init__(self, sysctl_conf, sysctl_d_confs_usr, sysctl_d_confs_etc):
        super(SysctlConfs, self).__init__()
        if sysctl_d_confs_usr:
            # Sort based on the filename to make sure
            # entries are overridden in the correct order.
            sysctl_d_confs_usr = sorted(sysctl_d_confs_usr, key=lambda x: x.file_name)

            for conf in sysctl_d_confs_usr:
                self.update(conf)

        if sysctl_d_confs_etc:
            # Sort based on the filename to make sure
            # entries are overridden in the correct order.
            sysctl_d_confs_etc = sorted(sysctl_d_confs_etc, key=lambda x: x.file_name)

            for conf in sysctl_d_confs_etc:
                self.update(conf)

        if sysctl_conf:
            self.update(sysctl_conf)

    def search(self, s_word):
        found = dict()
        for key, val in self.items():
            if re.search(s_word, key):
                found[key] = val

        return found



"""
CinderApiLog - file ``/var/log/cinder/cinder-api.log``
======================================================

CinderVolumeLog - file ``/var/log/cinder/volume.log``
=====================================================

This is a standard log parser based on the LogFileOutput class.

Sample input::

    2015-06-19 07:31:41.020 7947 DEBUG cinder.openstack.common.periodic_task [-] Running periodic task VolumeManager._publish_service_capabilities run_periodic_tasks /usr/lib/python2.7/site-packages/cinder/openstack/common/periodic_task.py:178
    2015-06-19 07:31:42.220 7947 DEBUG cinder.manager [-] Notifying Schedulers of capabilities ... _publish_service_capabilities /usr/lib/python2.7/site-packages/cinder/manager.py:128
    2015-06-19 07:31:47.319 7947 DEBUG cinder.openstack.common.periodic_task [-] Running periodic task VolumeManager._report_driver_status run_periodic_tasks /usr/lib/python2.7/site-packages/cinder/openstack/common/periodic_task.py:178
    2015-06-19 07:32:53.612 7947 INFO cinder.volume.manager [-] Updating volume status

Examples:

    >>> logs = shared[CinderVolumeLog]
    >>> 'Updating volume status' in logs
    True
    >>> logs.get('cinder.openstack.common.periodic_task')
    ['2015-06-19 07:31:41.020 7947 DEBUG cinder.openstack.common.periodic_task [-] Running periodic task VolumeManager._publish_service_capabilities run_periodic_tasks /usr/lib/python2.7/site-packages/cinder/openstack/common/periodic_task.py:178',
     '2015-06-19 07:31:47.319 7947 DEBUG cinder.openstack.common.periodic_task [-] Running periodic task VolumeManager._report_driver_status run_periodic_tasks /usr/lib/python2.7/site-packages/cinder/openstack/common/periodic_task.py:178']
    >>> from datetime import datetime
    >>> logs.get_after(datetime(2015, 6, 19, 7, 32, 0))
    ['2015-06-19 07:32:53.612 7947 INFO cinder.volume.manager [-] Updating volume status']
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.cinder_api_log)
class CinderApiLog(LogFileOutput):
    """Class for parsing the ``/var/log/cinder/cinder-api.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`
    """
    pass


@parser(Specs.cinder_volume_log)
class CinderVolumeLog(LogFileOutput):
    """
    Provide access to Cinder volume logs using the LogFileOutput parser class.

    .. note::
        Please refer to the super-class :class:`insights.core.LogFileOutput`
    """
    pass



from insights.parsers.glance_log import GlanceApiLog
from insights.tests import context_wrap

from datetime import datetime

API_LOG = """
2016-11-09 14:36:35.618 26656 WARNING oslo_config.cfg [-] Option "rpc_backend" from group "DEFAULT" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:36:38.717 26656 INFO glance.common.wsgi [-] Starting 4 workers
2016-11-09 14:36:38.737 27285 INFO eventlet.wsgi.server [-] (27285) wsgi starting up on http://172.18.0.13:9292
2016-11-09 14:36:38.737 26656 INFO glance.common.wsgi [-] Started child 27285
"""


def test_glance_api_log():
    log = GlanceApiLog(context_wrap(API_LOG))
    assert len(log.get('INFO')) == 3
    assert len(list(log.get_after(datetime(2016, 11, 9, 14, 36, 38)))) == 3



from insights.parsers.sysconfig import NetconsoleSysconfig
from insights.tests import context_wrap


netconsole = '''
# This is the configuration file for the netconsole service.  By starting
# this service you allow a remote syslog daemon to record console output
# from this system.

# The local port number that the netconsole module will use
LOCALPORT=6666

# The ethernet device to send console messages out of (only set this if it
# can't be automatically determined)
# DEV=

# The IP address of the remote syslog server to send messages to
# SYSLOGADDR=

# The listening port of the remote syslog daemon
SYSLOGPORT=514

# The MAC address of the remote syslog server (only set this if it can't
# be automatically determined)
# SYSLOGMACADDR=
'''.strip()


def test_netconsole():
    result = NetconsoleSysconfig(context_wrap(netconsole))
    assert result["LOCALPORT"] == '6666'
    assert result.get("DEV") is None
    assert result['SYSLOGPORT'] == "514"



from insights.parsers import pulp_worker_defaults
from insights.tests import context_wrap

import doctest

pulp_worker_defaults_in_docs = '''
# Configuration file for Pulp's Celery workers

# Define the number of worker nodes you wish to have here. This defaults to the number of processors
# that are detected on the system if left commented here.
PULP_CONCURRENCY=1

# Configure Python's encoding for writing all logs, stdout and stderr
PYTHONIOENCODING="UTF-8"
'''


def test_pulp_worker_defaults_docs():
    env = {
        'PulpWorkerDefaults': pulp_worker_defaults.PulpWorkerDefaults,
        'shared': {
            pulp_worker_defaults.PulpWorkerDefaults: pulp_worker_defaults.PulpWorkerDefaults(
                context_wrap(pulp_worker_defaults_in_docs)
            )
        },
    }
    failed, total = doctest.testmod(pulp_worker_defaults, globs=env)
    assert failed == 0



"""
Nginx Configurations
====================

This module includes the following parsers:

NginxConf - file ``/etc/nginx/nginx.conf`` and other Nginx configuration files
------------------------------------------------------------------------------

NginxConfPEG - file ``/etc/nginx/nginx.conf`` and other Nginx configuration files
---------------------------------------------------------------------------------

ContainerNginxConfPEG - file ``/etc/nginx/nginx.conf`` and other Nginx configuration files of running containers
----------------------------------------------------------------------------------------------------------------
"""
import string

from insights.contrib.nginxparser import create_parser, UnspacedList
from insights.core import ConfigParser, LegacyItemAccess, Parser, ContainerParser
from insights.core.plugins import parser
from insights.parsers import ParseException, get_active_lines
from insights.parsr import (EOF, EmptyQuotedString, Forward, LeftCurly, Lift, LineEnd, RightCurly,
                            Many, Number, OneLineComment, PosMarker, SemiColon, QuotedString,
                            skip_none, String, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.nginx_conf)
class NginxConf(Parser, LegacyItemAccess):
    """
    .. warning::
        This parser is deprecated, please import
        :py:class:`insights.combiners.nginx_conf.NginxConfTree` instead.

    Parse the keyword-and-value of a Nginx configuration file.

    Generally, each line is split on the first space into key and value, leading
    and trailing space being ignored.

    Example nginx.conf file::

        user       root
        worker_processes  5;
        error_log  logs/error.log;
        pid        logs/nginx.pid;
        worker_rlimit_nofile 8192;

        events {
          worker_connections  4096;
        }

        mail {
          server_name mail.example.com;
          auth_http  localhost:9000/cgi-bin/auth;
          server {
            listen   143;
            protocol imap;
          }
        }

        http {
          include  /etc/nginx/conf.d/*.conf
          index    index.html index.htm index.php;

          default_type application/octet-stream;
          log_format   main '$remote_addr - $remote_user [$time_local]  $status '
                            '"$request" $body_bytes_sent "$http_referer" '
                            '"$http_user_agent" "$http_x_forwarded_for"';
          access_log   logs/access.log  main;
          sendfile     on;
          tcp_nopush   on;
          server_names_hash_bucket_size 128;

          server { # php/fastcgi
            listen       80;
            server_name  domain1.com www.domain1.com;
            access_log   logs/domain1.access.log  main;
            root         html;

            location ~ \.php$ {
              fastcgi_pass   127.0.0.1:1025;
            }
          }

          server { # simple reverse-proxy
            listen       80;
            server_name  domain2.com www.domain2.com;
            access_log   logs/domain2.access.log  main;

            location ~ ^/(images|javascript|js|css|flash|media|static)/  {
              root    /var/www/virtual/big.server.com/htdocs;
              expires 30d;
            }

            location / {
              proxy_pass   http://127.0.0.1:8080;
            }
          }

          map $http_upgrade $connection_upgrade {
            default upgrade;
            '' close;
          }

          upstream websocket {
            server 10.66.208.205:8010;
          }

          upstream big_server_com {
            server 127.0.0.3:8000 weight=5;
            server 127.0.0.3:8001 weight=5;
            server 192.168.0.1:8000;
            server 192.168.0.1:8001;
          }

          server { # simple load balancing
            listen          80;
            server_name     big.server.com;
            access_log      logs/big.server.access.log main;

            location / {
              proxy_pass      http://big_server_com;
            }
          }
        }

    Examples:
        >>> type(nginxconf)
        <class 'insights.parsers.nginx_conf.NginxConf'>
        >>> nginxconf['user']
        'root'
        >>> nginxconf['events']['worker_connections'] # Values are all kept as strings.
        '4096'
        >>> nginxconf['mail']['server'][0]['listen']
        '143'
        >>> nginxconf['http']['access_log']
        'logs/access.log  main'
        >>> nginxconf['http']['server'][0]['location'][0]['fastcgi_pass']
        '127.0.0.1:1025'
    """
    def __init__(self, *args, **kwargs):
        deprecated(NginxConf, "Import NginxConfTree from insights.combiners.nginx_conf instead.", "3.0.300")
        super(NginxConf, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        list_result = UnspacedList(create_parser().parseString("\n".join(get_active_lines(content))).asList())
        self.data = self._convert_nginx_list_to_dict(list_result)

    def _convert_nginx_list_to_dict(self, li):
        """
        After parsed by create_parser(), the result is a list, it is better to convert to dict for convenience.
        """

        def _list_depth_to_dict(self, li):
            """
            Function to convert list whose depth is tree to dict. Generally, the section name would be a dict key, and content
            embraced by brace would be value. In some sections, the first item is like ['location', '/'], in this case, the
            convert rule is that 'location' would be the key, and add {"name":'/'} key-value to the dict value.
            """
            dict_ret = {}
            if self._depth(li) == 1:
                return self._handle_key_value(dict_ret, li[0], li[1])
            else:
                for sub_item in li[1]:
                    tmp_dict = _list_depth_to_dict(self, sub_item)
                    tmp_key = list(tmp_dict.keys())[0]
                    tmp_val = tmp_dict[tmp_key]
                    tmp_val = tmp_val[0] if isinstance(tmp_val, list) else tmp_val
                    dict_ret.update(self._handle_key_value(dict_ret, tmp_key, tmp_val))
                if len(li[0]) > 1:
                    dict_ret["name"] = ' '.join(li[0][1:])
                return {li[0][0]: dict_ret}

        dict_all = {}
        for item in li:
            dict_all.update(_list_depth_to_dict(self, item))
        return dict_all

    def _handle_key_value(self, t_dict, key, value):
        """
        Function to handle dict key has multi value, and return the values as list.
        """
        # As it is possible that key "server", "location", "include"
        # and "upstream" have multi value, set the value of dict as list.
        if key in ("server", "location", "include", "upstream"):
            if key in t_dict:
                val = t_dict[key]
                val.append(value)
                return {key: val}
            return {key: [value]}
        else:
            return {key: value}

    def _depth(self, l):
        """
        Function to count the depth of a list
        """
        if isinstance(l, list) and len(l) > 0:
            return 1 + max(self._depth(item) for item in l)
        else:
            return 0


@parser(Specs.nginx_conf)
class NginxConfPEG(ConfigParser):
    """
    Parse the keyword-and-value of a Nginx configuration file.

    Example nginx.conf file::

        user       root
        worker_processes  5;
        error_log  logs/error.log;
        pid        logs/nginx.pid;
        worker_rlimit_nofile 8192;

        events {
          worker_connections  4096;
        }

        mail {
          server_name mail.example.com;
          auth_http  localhost:9000/cgi-bin/auth;
          server {
            listen   143;
            protocol imap;
          }
        }

        http {
          include  /etc/nginx/conf.d/*.conf
          index    index.html index.htm index.php;

          default_type application/octet-stream;
          log_format   main '$remote_addr - $remote_user [$time_local]  $status '
                            '"$request" $body_bytes_sent "$http_referer" '
                            '"$http_user_agent" "$http_x_forwarded_for"';
          access_log   logs/access.log  main;
          sendfile     on;
          tcp_nopush   on;
          server_names_hash_bucket_size 128;

          server { # php/fastcgi
            listen       80;
            server_name  domain1.com www.domain1.com;
            access_log   logs/domain1.access.log  main;
            root         html;

            location ~ \.php$ {
              fastcgi_pass   127.0.0.1:1025;
            }
          }

          server { # simple reverse-proxy
            listen       80;
            server_name  domain2.com www.domain2.com;
            access_log   logs/domain2.access.log  main;

            location ~ ^/(images|javascript|js|css|flash|media|static)/  {
              root    /var/www/virtual/big.server.com/htdocs;
              expires 30d;
            }

            location / {
              proxy_pass   http://127.0.0.1:8080;
            }
          }

          map $http_upgrade $connection_upgrade {
            default upgrade;
            '' close;
          }

          upstream websocket {
            server 10.66.208.205:8010;
          }

          upstream big_server_com {
            server 127.0.0.3:8000 weight=5;
            server 127.0.0.3:8001 weight=5;
            server 192.168.0.1:8000;
            server 192.168.0.1:8001;
          }

          server { # simple load balancing
            listen          80;
            server_name     big.server.com;
            access_log      logs/big.server.access.log main;

            location / {
              proxy_pass      http://big_server_com;
            }
          }
        }

    Examples:
        >>> type(nginxconfpeg)
        <class 'insights.parsers.nginx_conf.NginxConfPEG'>
        >>> nginxconfpeg['user'][-1].value
        'root'
        >>> nginxconfpeg['events']['worker_connections'][-1].value
        4096
        >>> nginxconfpeg['mail']['server'][0]['listen'][-1].value
        143
        >>> nginxconfpeg['http']['access_log'][-1].value
        'logs/access.log main'
        >>> nginxconfpeg['http']['server'][0]['location'][0]['fastcgi_pass'][-1].value
        '127.0.0.1:1025'
    """
    def __init__(self, *args, **kwargs):
        def to_entry(name, attrs, body):
            if body == ";":
                return Directive(name=name.value, attrs=attrs, lineno=name.lineno, src=self)
            return Section(name=name.value, attrs=attrs, children=body, lineno=name.lineno, src=self)

        name_chars = string.ascii_letters + "_/"
        Stmt = Forward()
        Num = Number & (WSChar | LineEnd | SemiColon)
        Comment = OneLineComment("#").map(lambda x: None)
        BeginBlock = WS >> LeftCurly << WS
        EndBlock = WS >> RightCurly << WS
        Bare = String(set(string.printable) - (set(string.whitespace) | set("#;{}'\"")))
        Name = WS >> PosMarker(String(name_chars) | EmptyQuotedString(name_chars)) << WS
        Attr = WS >> (Num | Bare | QuotedString) << WS
        Attrs = Many(Attr)
        Block = BeginBlock >> Many(Stmt).map(skip_none) << EndBlock
        Stanza = (Lift(to_entry) * Name * Attrs * (Block | SemiColon)) | Comment
        Stmt <= WS >> Stanza << WS
        Doc = Many(Stmt).map(skip_none)
        self.Top = Doc + EOF
        super(NginxConfPEG, self).__init__(*args, **kwargs)

    def parse_doc(self, content):
        try:
            return Entry(children=self.Top("\n".join(content))[0], src=self)
        except Exception:
            raise ParseException("There was an exception when parsing the config file.")


@parser(Specs.container_nginx_conf)
class ContainerNginxConfPEG(ContainerParser, NginxConfPEG):
    """
    Parser for the Nginx configuration files of running container.
    """
    pass



from insights.parsers import saphostctrl, ParseException, SkipException
from insights.parsers.saphostctrl import SAPHostCtrlInstances
from insights.tests import context_wrap
import doctest
import pytest

SAPHOSTCTRL_HOSTINSTANCES_DOCS = '''
*********************************************************
 SID , String , D89
 SystemNumber , String , 88
 InstanceName , String , HDB88
 InstanceType , String , HANA Test
 Hostname , String , lu0417
 FullQualifiedHostname , String , lu0417.example.com
 IPAddress , String , 10.0.0.88
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 SID , String , D90
 SystemNumber , String , 90
 InstanceName , String , HDB90
 InstanceType , String , HANA Test
 Hostname , String , lu0418
 FullQualifiedHostname , String , lu0418.example.com
 IPAddress , String , 10.0.0.90
 SapVersionInfo , String , 749, patch 211, changelist 1754007
'''

SAPHOSTCTRL_HOSTINSTANCES_DOCS_OLD = '''
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D89
 SystemNumber , String , 88
 InstanceType , String , HANA Test
 InstanceName , String , HDB88
 Hostname , String , lu0417
 FullQualifiedHostname , String , lu0417.example.com
 IPAddress , String , 10.0.0.88
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 CreationClassName , String , SAPInstance
 SID , String , D90
 SystemNumber , String , 90
 InstanceName , String , HDB90
 Hostname , String , lu0418
 FullQualifiedHostname , String , lu0418.example.com
 IPAddress , String , 10.0.0.90
 SapVersionInfo , String , 749, patch 211, changelist 1754007
'''

SAPHOSTCTRL_HOSTINSTANCES_GOOD = '''
*********************************************************
 SID , String , D89
 SystemNumber , String , 88
 InstanceName , String , HDB88
 InstanceType , String , HANA
 Hostname , String , li-ld-1810
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.0.1
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 SID , String , D90
 SystemNumber , String , 90
 InstanceName , String , HDB90
 InstanceType , String , HANA
 Hostname , String , li-ld-1810
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.0.1
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 SID , String , D79
 SystemNumber , String , 08
 InstanceName , String , ERS08
 InstanceType , String , Enqueue Replication Server
 Hostname , String , d79ers
 FullQualifiedHostname , String , d79ers.example.com
 IPAddress , String , 10.0.0.15
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 SID , String , D79
 SystemNumber , String , 07
 InstanceName , String , ASCS07
 InstanceType , String , ABAP Central Services
 Hostname , String , d79ascs
 FullQualifiedHostname , String , d79ascs.example.com
 IPAddress , String , 10.0.0.14
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 SID , String , D79
 SystemNumber , String , 09
 InstanceName , String , DVEBMGS09
 InstanceType , String , Primary Application Server
 Hostname , String , d79pas
 FullQualifiedHostname , String , d79pas.example.com
 IPAddress , String , 10.0.0.16
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 SID , String , D80
 SystemNumber , String , 10
 InstanceName , String , SCS10
 InstanceType , String , Java Central Services
 Hostname , String , d80scs
 FullQualifiedHostname , String , d80scs.example.com
 IPAddress , String , 10.0.0.17
 SapVersionInfo , String , 749, patch 301, changelist 1779613
*********************************************************
 SID , String , D62
 SystemNumber , String , 62
 InstanceName , String , HDB62
 InstanceType , String , HANA
 Hostname , String , d62dbsrv
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.1.12
 SapVersionInfo , String , 749, patch 211, changelist 1754007
*********************************************************
 SID , String , D52
 SystemNumber , String , 52
 InstanceName , String , ASCS52
 InstanceType , String , ABAP Central Services
 Hostname , String , d52ascs
 FullQualifiedHostname , String , d52ascs.example.com
 IPAddress , String , 10.0.0.20
 SapVersionInfo , String , 749, patch 401, changelist 1806777
*********************************************************
 SID , String , D52
 SystemNumber , String , 54
 InstanceName , String , D54
 InstanceType , String , ABAP Dialog Instance
 Hostname , String , d52pas
 FullQualifiedHostname , String , d52pas.example.com
 IPAddress , String , 10.0.0.22
 SapVersionInfo , String , 749, patch 401, changelist 1806777
*********************************************************
 SID , String , SMA
 SystemNumber , String , 91
 InstanceName , String , SMDA91
 InstanceType , String , Solution Manager Diagnostic Agent
 Hostname , String , li-ld-1810
 FullQualifiedHostname , String , li-ld-1810.example.com
 IPAddress , String , 10.0.0.1
 SapVersionInfo , String , 749, patch 200, changelist 1746260
*********************************************************
 SID , String , B15
 SystemNumber , String , 00
 InstanceName , String , HDB00
 InstanceType , String , HANA
 Hostname , String , sapb15hdba1
 FullQualifiedHostname , String , li-ld-1810.example.com
 SapVersionInfo , String , 749, patch 418, changelist 1816226
'''

SAPHOSTCTRL_HOSTINSTANCES_BAD = '''
 SID , String , D89
 SystemNumber , String , 88
 InstanceName , String
'''


def test_saphostctrl_docs():
    globs = {
        'sap_inst': SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_DOCS))
    }
    failed, total = doctest.testmod(saphostctrl, globs=globs)
    assert failed == 0


def test_saphostctrl():
    sap = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_GOOD))
    assert len(sap) == 11
    assert sap.data[-3]['SapVersionInfo'] == '749, patch 401, changelist 1806777'
    assert sap[-3]['SapVersionInfo'] == '749, patch 401, changelist 1806777'
    assert sorted(sap.instances) == sorted([
        'HDB88', 'HDB90', 'ERS08', 'ASCS07', 'DVEBMGS09', 'SCS10', 'HDB62',
        'HDB00', 'ASCS52', 'D54', 'SMDA91'
    ])
    for sid in ['D89', 'D90', 'D79', 'D80', 'D62', 'D52', 'SMA']:
        assert sid in sap.sids
    assert sorted(sap.types) == sorted([
        'HDB', 'ERS', 'ASCS', 'DVEBMGS', 'SCS', 'D', 'SMDA'
    ])


def test_saphostctrl_old():
    sap = SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_DOCS_OLD))
    assert sap.types == ['HDB']  # short types only
    assert sap[0]['InstanceType'] == 'HANA Test'
    assert sap[1]['InstanceType'] == 'HDB'


def test_saphostctrl_bad():
    with pytest.raises(SkipException):
        SAPHostCtrlInstances(context_wrap(''))

    with pytest.raises(ParseException):
        SAPHostCtrlInstances(context_wrap(SAPHOSTCTRL_HOSTINSTANCES_BAD))



"""
EtcdConf - file ``/etc/etcd/etcd.conf``
=======================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.core.filters import add_filter
from insights.specs import Specs

add_filter(Specs.etcd_conf, ["["])


@parser(Specs.etcd_conf)
class EtcdConf(IniConfigFile):
    """
    The EtcdConf class parses the file ``/etc/etcd/etcd.conf``. The
    etcd.conf is in the standard 'ini' format and is read by the base
    parser class `IniConfigFile`.

    Typical contents of the file look like::

        [member]
        ETCD_NAME=f05-h19-000-1029p.rdu2.scalelab.redhat.com
        ETCD_LISTEN_PEER_URLS=https://10.1.40.235:2380
        ETCD_DATA_DIR=/var/lib/etcd/
        ETCD_HEARTBEAT_INTERVAL=500
        ETCD_ELECTION_TIMEOUT=2500
        ETCD_LISTEN_CLIENT_URLS=https://10.1.40.235:2379

        [auth]
        ETCD_AUTH_TOKEN=simple

    Examples:
        >>> type(conf)
        <class 'insights.parsers.etcd_conf.EtcdConf'>
        >>> conf.get('auth', 'ETCD_AUTH_TOKEN') == 'simple'
        True
        >>> conf.has_option('member', 'ETCD_NAME')
        True
    """
    pass



"""
Octavia - file ``octavia.conf``
===============================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.core.filters import add_filter
from insights.specs import Specs

VALID_KEYS = [
    '[',
    'active_connection_max_retries',
    'active_connection_rety_interval',
    'admin_log_targets',
    'administrative_log_facility',
    'agent_request_read_timeout',
    'agent_server_ca',
    'agent_server_cert',
    'agent_server_network_dir',
    'agent_server_network_file',
    'agent_tls_protocol',
    'allow_pagination',
    'allow_ping_health_monitors',
    'allow_sorting',
    'allow_tls_terminated_listeners',
    'allow_vip_network_id',
    'allow_vip_port_id',
    'allow_vip_subnet_id',
    'amp_active_retries',
    'amp_active_wait_sec',
    'amp_boot_network_list',
    'amp_flavor_id',
    'amp_image_id',
    'amp_image_owner_id',
    'amp_image_tag',
    'amp_secgroup_list',
    'amp_ssh_access_allowed',
    'amphora_driver',
    'amphora_expiry_age',
    'amphora_udp_driver',
    'anti_affinity_policy',
    'api_base_uri',
    'audit_map_file',
    'auth_strategy',
    'auth_type',
    'availability_zone',
    'base_cert_dir',
    'base_path',
    'bind_host',
    'bind_ip',
    'bind_port',
    'build_active_retries',
    'build_rate_limit',
    'build_retry_interval',
    'ca_certificates_file',
    'cafile',
    'cert_generator',
    'cert_manager',
    'cert_validity_time',
    'cleanup_interval',
    'client_ca',
    'client_cert',
    'compute_driver',
    'connection_logging',
    'connection_max_retries',
    'connection_retry_interval',
    'controller_ip_port_list',
    'debug',
    'default_health_monitor_quota',
    'default_listener_quota',
    'default_load_balancer_quota',
    'default_member_quota',
    'default_pool_quota',
    'default_provider_driver',
    'disable_local_log_storage',
    'disable_revert',
    'distributor_driver',
    'driver',
    'enable_anti_affinity',
    'enable_proxy_headers_parsing',
    'enabled',
    'enabled_provider_agents',
    'enabled_provider_drivers',
    'endpoint_type',
    'engine',
    'failover_threads',
    'forward_all_logs',
    'get_socket_path',
    'graceful_shutdown_timeout',
    'haproxy_cmd',
    'haproxy_stick_size',
    'haproxy_template',
    'health_check_interval',
    'health_update_driver',
    'health_update_threads',
    'heartbeat_interval',
    'heartbeat_timeout',
    'ignore_req_list',
    'insecure',
    'lb_network_interface',
    'load_balancer_expiry_age',
    'loadbalancer_topology',
    'log_dir',
    'log_file',
    'log_protocol',
    'log_queue_size',
    'log_retry_count',
    'log_retry_interval',
    'logging_template_override',
    'max_process_warning_percent',
    'max_retries',
    'max_workers',
    'memcached_servers',
    'network_driver',
    'octavia_plugins',
    'pagination_max_limit',
    'policy_file',
    'port_detach_timeout',
    'provider_agent_shutdown_timeout',
    'random_amphora_name_length',
    'region_name',
    'respawn_count',
    'respawn_interval',
    'rest_request_conn_timeout',
    'rest_request_read_timeout',
    'retry_interval',
    'rpc_thread_pool_size',
    'server_ca',
    'service_name',
    'signing_digest',
    'sock_rlimit',
    'spare_amphora_pool_size',
    'spare_check_interval',
    'stats_max_processes',
    'stats_request_timeout',
    'stats_socket_path',
    'stats_update_driver',
    'stats_update_threads',
    'status_max_processes',
    'status_request_timeout',
    'status_socket_path',
    'status_update_threads',
    'storage_path',
    'tenant_log_targets',
    'topic',
    'topics',
    'udp_connect_min_interval_health_monitor',
    'use_oslo_messaging',
    'use_upstart',
    'user_data_config_drive',
    'user_log_facility',
    'user_log_format',
    'volume_create_max_retries',
    'volume_create_retry_interval',
    'volume_create_timeout',
    'volume_driver',
    'volume_size',
    'volume_type',
    'vrrp_advert_int',
    'vrrp_check_interval',
    'vrrp_fail_count',
    'vrrp_garp_refresh_count',
    'vrrp_garp_refresh_interval',
    'vrrp_success_count',
    'workers'
]

add_filter(Specs.octavia_conf, VALID_KEYS)


@parser(Specs.octavia_conf)
class OctaviaConf(IniConfigFile):
    """
    Provides a parser arser for file
    ``/var/lib/config-data/puppet-generated/octavia/etc/octavia/octavia.conf``.  Filters
    have been added to this parser to ensure that the necessary data will be collected.

    Sample input data::

        [DEFAULT]
        # Print debugging output (set logging level to DEBUG instead of default WARNING level).
        debug = False

        [api_settings]
        bind_host = 127.0.0.1
        bind_port = 9876
        # Default provider driver
        default_provider_driver = amphora

        # The minimum health monitor delay interval for UDP-CONNECT Health Monitor type
        udp_connect_min_interval_health_monitor = 3

    Examples:
        >>> type(octavia_conf)
        <class 'insights.parsers.octavia.OctaviaConf'>
        >>> octavia_conf.defaults()['debug'] == 'False'
        True
        >>> octavia_conf.get('api_settings', 'bind_port') == '9876'
        True
        >>> octavia_conf.has_option('api_settings', 'missing_key')
        False
    """
    pass



import doctest
import pytest
from insights.parsers import dotnet
from insights.core.plugins import ContentException
from insights.parsers import SkipException, ParseException
from insights.parsers.dotnet import DotNetVersion, ContainerDotNetVersion
from insights.tests import context_wrap

dotnet_version_1 = "3.1.108"
dotnet_version_2 = "2.1.518"
dotnet_version_3 = """
-bash: /usr/bin/dotnet: No such file or directory
""".strip()
dotnet_version_4 = "2."
dotnet_version_5 = """
abc
-bash: /usr/bin/dotnet: No such file or directory
"""


def test_dotnet_version():
    ret = DotNetVersion(context_wrap(dotnet_version_1))
    assert ret.major == 3
    assert ret.minor == 1
    assert ret.raw == dotnet_version_1

    ret = DotNetVersion(context_wrap(dotnet_version_2))
    assert ret.major == 2
    assert ret.minor == 1
    assert ret.raw == dotnet_version_2


def test_dotnet_version_ab():
    with pytest.raises(ContentException):
        ret = DotNetVersion(context_wrap(dotnet_version_3))
        assert ret is None

    with pytest.raises(ParseException) as pe:
        ret = DotNetVersion(context_wrap(dotnet_version_4))
        assert ret is None
        assert "Unrecognized version" in str(pe)

    with pytest.raises(SkipException):
        ret = DotNetVersion(context_wrap(dotnet_version_5))
        assert ret is None


def test_container_dotnet_version():
    ret = ContainerDotNetVersion(
        context_wrap(
            dotnet_version_1,
            container_id='cc2883a1a369',
            image='quay.io/rhel8',
            engine='podman'
        )
    )
    assert ret.major == 3
    assert ret.minor == 1
    assert ret.raw == dotnet_version_1
    assert ret.image == "quay.io/rhel8"
    assert ret.engine == "podman"
    assert ret.container_id == "cc2883a1a369"


def test_container_dotnet_version_error():
    with pytest.raises(ContentException):
        ret = ContainerDotNetVersion(
            context_wrap(
                dotnet_version_3,
                container_id='cc2883a1a369',
                image='quay.io/rhel8',
                engine='podman'
            )
        )
        assert ret is None


def test_doc_examples():
    env = {
        'dotnet_ver': DotNetVersion(context_wrap(dotnet_version_1)),
        'con_dotnet_ver': ContainerDotNetVersion(
            context_wrap(
                dotnet_version_1,
                container_id='cc2883a1a369',
                image='quay.io/rhel8',
                engine='podman'
            )
        )
    }
    failed, total = doctest.testmod(dotnet, globs=env)
    assert failed == 0



"""
Custom datasource for collecting yum updates
"""
import json
import time

from insights import datasource, HostContext, SkipComponent
from insights.components.rhel_version import IsRhel7, IsRhel8, IsRhel9
from insights.core.spec_factory import DatasourceProvider
from distutils.version import LooseVersion as version

try:
    from functools import cmp_to_key

    # cmp_to_key is not available in python 2.6, but it has sorted function which accepts cmp function
    def sorted_cmp(it, cmp):
        return sorted(it, key=cmp_to_key(cmp))
except ImportError:
    sorted_cmp = sorted


class DnfManager:
    """ Performs package resolution on dnf based systems """
    def __init__(self):
        self.base = dnf.base.Base()
        self.releasever = dnf.rpm.detect_releasever("/")
        self.basearch = dnf.rpm.basearch(hawkey.detect_arch())
        self.packages = []
        self.repos = []

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass

    @staticmethod
    def pkg_cmp(a, b):
        if a.name != b.name:
            return -1 if a.name < b.name else 1
        vercmp = rpm.labelCompare((str(a.e), a.v, a.r), (str(b.e), b.v, b.r))
        if vercmp != 0:
            return vercmp
        if a.reponame != b.reponame:
            return -1 if a.reponame < b.reponame else 1
        return 0

    def sorted_pkgs(self, pkgs):
        # if package is installed more than once (e.g. kernel)
        # don't report other installed (i.e. with @System repo) as updates
        return sorted_cmp([pkg for pkg in pkgs if pkg.reponame != "@System"], self.pkg_cmp)

    def load(self):
        self.base.conf.read()
        self.base.conf.cacheonly = True
        self.base.read_all_repos()
        self.packages = hawkey.Query(hawkey.Sack())
        if version(dnf.VERSION) >= version("4.7.0"):
            try:
                # do not use standard self.base.fill_sack() it does not respect cacheonly
                self.base.fill_sack_from_repos_in_cache(load_system_repo=True)
                self.packages = self.base.sack.query()
            except dnf.exceptions.RepoError:
                # RepoError is raised when cache is empty
                pass
        self.repos = self.base.repos

    def installed_packages(self):
        return self.packages.installed().run()

    def updates(self, pkg):
        name = pkg.name
        evr = "{0}:{1}-{2}".format(pkg.epoch, pkg.version, pkg.release)
        arch = pkg.arch
        nevra = "{0}-{1}.{2}".format(name, evr, arch)
        updates_list = []
        for upd in self.packages.filter(name=name, arch=arch, evr__gt=evr):
            updates_list.append(upd)
        return nevra, updates_list

    @staticmethod
    def pkg_nevra(pkg):
        return "{0}-{1}:{2}-{3}.{4}".format(pkg.name, pkg.epoch, pkg.version, pkg.release, pkg.arch)

    @staticmethod
    def pkg_repo(pkg):
        return pkg.reponame

    @staticmethod
    def advisory(pkg):
        errata = pkg.get_advisories(hawkey.EQ)
        return errata[0].id if len(errata) > 0 else None

    def last_update(self):
        last_ts = 0
        for repo in self.base.repos.iter_enabled():
            repo_ts = repo._repo.getTimestamp()
            if repo_ts > last_ts:
                last_ts = repo_ts
        return last_ts


class YumManager(DnfManager):
    """ Performs package resolution on yum based systems """
    def __init__(self):
        self.base = yum.YumBase()
        self.base.doGenericSetup(cache=1)
        self.releasever = self.base.conf.yumvar['releasever']
        self.basearch = self.base.conf.yumvar['basearch']
        self.packages = []
        self.repos = []
        self.updict = {}

    @staticmethod
    def pkg_cmp(a, b):
        vercmp = a.verCMP(b)
        if vercmp != 0:
            return vercmp
        if a.repoid != b.repoid:
            return -1 if a.repoid < b.repoid else 1
        return 0

    def sorted_pkgs(self, pkgs):
        return sorted_cmp(pkgs, self.pkg_cmp)

    def load(self):
        self.base.doRepoSetup()
        try:
            self.base.doSackSetup()
            self.packages = self.base.pkgSack.returnPackages()
        except yum.Errors.RepoError:
            # RepoError is raised when cache is empty
            pass
        self.repos = self.base.repos.repos
        self._build_updict()

    def _build_updict(self):
        self.updict = {}
        for pkg in self.packages:
            self.updict.setdefault(pkg.na, []).append(pkg)

    def installed_packages(self):
        return self.base.rpmdb.returnPackages()

    def updates(self, pkg):
        nevra = pkg.nevra
        updates_list = []
        for upg in self.updict.get(pkg.na, []):
            if upg.verGT(pkg):
                updates_list.append(upg)
        return nevra, updates_list

    @staticmethod
    def pkg_repo(pkg):
        return pkg.repoid

    def advisory(self, pkg):
        adv = self.base.upinfo.get_notice(pkg.nvr)
        if adv:
            return adv.get_metadata()['update_id']
        return None

    @staticmethod
    def last_update():
        return 0


# Select which manager to use based on the available system libraries.
try:
    import dnf
    import hawkey
    import rpm
    UpdatesManager = DnfManager
except ImportError:
    try:
        import yum
        UpdatesManager = YumManager
    except ImportError:
        UpdatesManager = None


@datasource(HostContext, [IsRhel7, IsRhel8, IsRhel9])
def yum_updates(_broker):
    """
    This datasource provides a list of available updates on the system.
    It uses the yum python library installed locally, and collects list of
    available package updates, along with advisory info where applicable.

    Sample data returned::

        {
          "releasever": "8",
          "basearch": "x86_64",
          "update_list": {
            "NetworkManager-1:1.22.8-4.el8.x86_64": {
              "available_updates": [
                {
                  "package": "NetworkManager-1:1.22.8-5.el8_2.x86_64",
                  "repository": "rhel-8-for-x86_64-baseos-rpms",
                  "basearch": "x86_64",
                  "releasever": "8",
                  "erratum": "RHSA-2020:3011"
                }
              ]
            }
          },
          "metadata_time": "2021-01-01T09:39:45Z"
        }

    Returns:
        list: List of available updates
    Raises:
        SkipComponent: Raised when neither dnf nor yum is found
    """
    if UpdatesManager is None:
        raise SkipComponent()

    with UpdatesManager() as umgr:
        umgr.load()

        response = {
            "releasever": umgr.releasever,
            "basearch": umgr.basearch,
            "update_list": {},
        }

        for pkg in umgr.installed_packages():
            nevra, updates_list = umgr.updates(pkg)
            if updates_list:
                out_list = []
                update_list = umgr.sorted_pkgs(updates_list)

                for p in update_list:
                    pkg_dict = {
                        "package": umgr.pkg_nevra(p),
                        "repository": umgr.pkg_repo(p),
                        "basearch": response["basearch"],
                        "releasever": response["releasever"],
                    }

                    erratum = umgr.advisory(p)
                    if erratum:
                        pkg_dict["erratum"] = erratum

                    out_list.append(pkg_dict)

                response["update_list"][nevra] = {"available_updates": out_list}

        ts = umgr.last_update()
        if ts:
            response["metadata_time"] = time.strftime("%FT%TZ", time.gmtime(ts))

    return DatasourceProvider(content=json.dumps(response), relative_path='insights_commands/yum_updates_list')



"""
HttpdM - command ``httpd -M``
=============================

Module for parsing the output of command ``httpd -M``.
"""

from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs
from insights.parsers import ParseException


@parser(Specs.httpd_M)
class HttpdM(LegacyItemAccess, CommandParser):
    """
    Class for parsing ``httpd -M`` command output.

    The data is kept in the ``data`` property and can be accessed through the
    object itself thanks to the ``LegacyItemAccess`` parser class.

    Typical output of command ``httpd -M`` looks like::

        Loaded Modules:
         core_module (static)
         http_module (static)
         access_compat_module (shared)
         actions_module (shared)
         alias_module (shared)
        Syntax OK

    Examples:
        >>> type(hm)
        <class 'insights.parsers.httpd_M.HttpdM'>
        >>> len(hm.loaded_modules)
        5
        >>> len(hm.static_modules)
        2
        >>> 'core_module' in hm.static_modules
        True
        >>> 'http_module' in hm.static_modules
        True
        >>> 'http_module' in hm
        True
        >>> 'http_module' in hm.shared_modules
        False
        >>> hm.httpd_command
        '/usr/sbin/httpd'

    Raise:
        ParseException: When input content is empty or there is no parsed data.

    Attributes:
        data (dict): All loaded modules are stored in this dictionary with the
                     name as the key and the loaded mode ('shared' or 'static')
                     as the value.
        loaded_modules (list): List of the loaded modules.
        static_modules (list): List of the loaded static modules.
        shared_modules (list): List of the loaded shared modules.
    """

    def parse_content(self, content):
        if not content:
            raise ParseException("Input content is empty.")

        self.data = {}
        self.loaded_modules = []
        self.static_modules = []
        self.shared_modules = []
        IGNORE_LINES = ('Loaded Modules:', 'Syntax')
        for line in content:
            if line.startswith((IGNORE_LINES)):
                continue
            line_splits = line.strip().split()
            if len(line_splits) == 2:
                module, mode = line_splits[0], line_splits[-1].strip("()")
                self.data[module] = mode

        if self.data:
            self.loaded_modules = self.data.keys()
            self.static_modules = [k for k, v in self.data.items() if v == 'static']
            self.shared_modules = [k for k, v in self.data.items() if v == 'shared']
        else:
            raise ParseException("Input content is not empty but there is no useful parsed data.")

    @property
    def httpd_command(self):
        """
        Return the full binary path of a running httpd or None when nothing
        is found. It's to identify which httpd binaries the instance run with.
        """
        return self.args



"""
RsyslogConfAll - files ``/etc/rsyslog.conf`` and ``/etc/rsyslog.d/*.conf``
==========================================================================

Combiner for accessing all the rsyslog comfiguration files. There may be
multiple rsyslog configuration, and the main configuration file is
``/etc/rsyslog.conf``. This combiner will not check same option in multi
files, user needs to check this situation in plugin if it is necessary.

"""
from insights.core.plugins import combiner
from insights.parsers.rsyslog_conf import RsyslogConf


@combiner(RsyslogConf)
class RsyslogAllConf(dict):
    """
    Combiner for accessing all the rsyslog configuration files.

    Examples:
        >>> type(confs)
        <class 'insights.combiners.rsyslog_confs.RsyslogAllConf'>
        >>> len(confs)
        2
        >>> confs['/etc/rsyslog.conf'][0]
        '$ModLoad imuxsock'
    """
    def __init__(self, confs):
        super(RsyslogAllConf, self).__init__()
        data = {}

        # Combine rsyslog configuration files into one dict. Key is file name, value is content of configuration file.
        for conf in confs:
            if conf.file_path == "/etc/rsyslog.conf":
                # Check if there is include option, if not, only parse /etc/rsyslog.conf even
                # /etc/rsyslog.d/*.conf exist.
                if not any(["include(" in item or "$IncludeConfig" in item for item in conf]):
                    data.clear()
                    data[conf.file_path] = conf
                    break
            data[conf.file_path] = conf

        self.update(data)



"""
``test parted``
===============
"""
import pytest

from insights.parsers import ParseException
from insights.parsers.parted import PartedL
from insights.tests import context_wrap

PARTED_DATA = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
 1      1049kB  525MB   524MB   primary  xfs          boot
 2      525MB   9664MB  9138MB  primary               lvm
""".strip()

PARTED_DATA_NO_SECTOR_SPLIT = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical/physical): 512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
 1      1049kB  525MB   524MB   primary  xfs          boot
 2      525MB   9664MB  9138MB  primary               lvm
"""

PARTED_DATA_2 = """
Model: IBM 2107900 (scsi)
Disk /dev/sdet: 2147MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type     File system  Flags
 1      32.3kB  2580kB  2548kB  primary
""".strip()

PARTED_DATA_3 = """
Model: DELL PERC H710 (scsi)
Disk /dev/sda: 292GB
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type      File system  Flags
 1      32.3kB  526MB   526MB   primary   ext3         boot
 2      526MB   9114MB  8587MB  primary   linux-swap
 3      9114MB  12.3GB  3224MB  primary   ext3
 4      12.3GB  292GB   280GB   extended
 5      12.3GB  254GB   241GB   logical   ext3
 6      254GB   281GB   26.8GB  logical   ext3
 7      281GB   285GB   4294MB  logical   ext3
 8      285GB   288GB   3224MB  logical   ext3
 9      288GB   290GB   2147MB  logical   ext3
10      290GB   292GB   2147MB  logical   ext3
""".strip()


def test_parted():
    context = context_wrap(PARTED_DATA)
    results = PartedL(context)
    assert results is not None
    assert results.get('model') == 'Virtio Block Device (virtblk)'
    assert results.disk == '/dev/vda'
    assert results.get('size') == '9664MB'
    assert results.get('sector_size') == '512B/512B'
    assert results.logical_sector_size == '512B'
    assert results.physical_sector_size == '512B'
    assert results.get('partition_table') == 'msdos'
    assert results.get('disk_flags') is None
    partitions = results.partitions
    assert len(partitions) == 2
    assert partitions[0].number == '1'
    assert partitions[0].start == '1049kB'
    assert partitions[0].end == '525MB'
    assert partitions[0].size == '524MB'
    assert partitions[0].file_system == 'xfs'
    assert partitions[0].get('name') is None
    assert partitions[0].type == 'primary'
    assert partitions[0].flags == 'boot'
    assert results.boot_partition is not None
    assert results.boot_partition.number == '1'

    assert partitions[1].get('file_system') == ''
    assert partitions[1].get('flags') == 'lvm'
    assert partitions[1].get('name') is None
    assert partitions[1].get('number') == '2'
    assert partitions[1].get('start') == '525MB'
    assert partitions[1].get('end') == '9664MB'
    assert partitions[1].get('size') == '9138MB'
    assert partitions[1].get('type') == 'primary'

    context = context_wrap(PARTED_DATA_2)
    results = PartedL(context)
    assert results is not None
    assert results.disk == '/dev/sdet'
    assert len(results.partitions) == 1

    context = context_wrap(PARTED_DATA_3)
    results = PartedL(context)
    assert results is not None
    assert results.disk == '/dev/sda'
    assert results.logical_sector_size == '512B'
    assert results.physical_sector_size == '512B'
    assert len(results.partitions) == 10


PARTED_ERR_DATA = """
Error: /dev/dm-1: unrecognised disk label
"""

PARTED_ERR_DATA_2 = """
Model: IBM 2107900 (scsi)
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type     File system  Flags
 1      32.3kB  2580kB  2548kB  primary
""".strip()

PARTED_ERR_DATA_NO_PARTITIONS = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

"""

PARTED_ERR_DATA_EXCEPTIONS = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical:physical): 512B:512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
"""


def test_failure_modes():
    context = context_wrap(PARTED_ERR_DATA)
    with pytest.raises(ParseException):
        PartedL(context)

    context = context_wrap(PARTED_ERR_DATA_2)
    with pytest.raises(ParseException):
        PartedL(context)

    # Test failure to find a slash in sector size
    results = PartedL(context_wrap(PARTED_DATA_NO_SECTOR_SPLIT))
    assert results is not None
    assert results.disk == '/dev/vda'
    assert results._sector_size is None
    assert results.logical_sector_size is None
    assert results.physical_sector_size is None

    # Object should be complete but has no data
    part = PartedL(context_wrap(PARTED_ERR_DATA_NO_PARTITIONS))
    assert part is not None
    assert part.disk == '/dev/vda'
    assert part.data['sector_size'] == '512B/512B'
    assert part.logical_sector_size == '512B'
    assert part.physical_sector_size == '512B'
    assert part.partitions == []

    # Various other failure modes that aren't actual parse exceptions atm
    part = PartedL(context_wrap(PARTED_ERR_DATA_EXCEPTIONS))
    assert part.disk == '/dev/vda'
    assert 'sector_size' not in part.data



"""
Component identifies Ceph Monitor
=================================

The ``Is*`` component in this module is valid if the
:py:class:`insights.combiners.ps.Ps` combiner indicates
the host is a Ceph monitor.  Otherwise, it raises a
:py:class:`insights.core.dr.SkipComponent` to prevent dependent components from
executing.

"""
from insights.core.dr import SkipComponent
from insights.core.plugins import component
from insights.combiners.ps import Ps


@component(Ps)
class IsCephMonitor(object):
    """
    This component uses ``Ps`` combiner to determine if the host is a Ceph
    monitor or not.  If not Ceph monitor, it raises ``SkipComponent``.

    Raises:
        SkipComponent: When it's not a Ceph monitor.
    """
    def __init__(self, ps):
        if not ps.search(COMMAND_NAME__contains='ceph-mon'):
            raise SkipComponent("Not Ceph Monitor")



"""
MssqlApiAssessment - file ``/var/opt/mssql/log/assessments/assessment-latest``
==============================================================================

Parsers contains in this module are:

MssqlApiAssessment - file ``/var/opt/mssql/log/assessments/assessment-latest``
"""

from insights import JSONParser, parser
from insights.specs import Specs


@parser(Specs.mssql_api_assessment)
class MssqlApiAssessment(JSONParser):
    """
    Parses the file: ``/var/opt/mssql/log/assessments/assessment-latest``

    Sample content of the file::

        [
          {
            "Timestamp": "2021-05-05T21:51:55.2317511-04:00",
            "Severity": "Information",
            "TargetType": "Server",
            "TargetName": "ceph4-mon",
            "TargetPath": "Server[@Name='ceph4-mon']",
            "CheckId": "TF174",
            "CheckName": "TF 174 increases plan cache bucket count",
            "Message": "Enable trace flag 174 to increase plan cache bucket count",
            "RulesetName": "Microsoft ruleset",
            "RulesetVersion": "1.0.305",
            "HelpLink": "https://docs.microsoft.com/sql/t-sql/database-console-commands/dbcc-traceon-trace-flags-transact-sql"
          },
          {
            "Timestamp": "2021-05-05T21:51:55.2323431-04:00",
            "Severity": "Information",
            "TargetType": "Server",
            "TargetName": "ceph4-mon",
            "TargetPath": "Server[@Name='ceph4-mon']",
            "CheckId": "TF834",
            "CheckName": "TF 834 enables large-page allocations",
            "Message": "Enable trace flag 834 to use large-page allocations to improve analytical and data warehousing workloads",
            "RulesetName": "Microsoft ruleset",
            "RulesetVersion": "1.0.305",
            "HelpLink": "https://support.microsoft.com/kb/3210239"
          }
        ]

    Examples:
        >>> type(mssql_api_assessment_output)
        <class 'insights.parsers.mssql_api_assessment.MssqlApiAssessment'>
        >>> mssql_api_assessment_output[0]["Severity"] == 'Information'
        True
    """
    pass



"""
OVSvsctlListBridge - command ``/usr/bin/ovs-vsctl list bridge``
===============================================================

This module provides class ``OVSvsctlListBridge`` for parsing the
output of command ``ovs-vsctl list bridge``.

.. note::
    Please refer to its super-class :class:`insights.parsers.ovs_vsctl_list.OVSvsctlList`
    for more details.

.. warning::
    This parser `OVSvsctlListBridge` is deprecated, please use
    :py:class:`insights.ovs_vsctl.OVSvsctlListBridge` instead.
"""

from insights import parser
from insights.parsers.ovs_vsctl import OVSvsctlList
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ovs_vsctl_list_bridge)
class OVSvsctlListBridge(OVSvsctlList):
    """
    .. warning::
        This parser `OVSvsctlListBridge` is deprecated, please use
        :py:class:`insights.ovs_vsctl.OVSvsctlListBridge` instead.

    Class to parse output of command ``ovs-vsctl list bridge``.
    Generally, the data is in key:value format with values having
    data types as string, numbers, list or dictionary.
    The class provides attribute ``data`` as list with lines parsed
    line by line based on keys for each bridge.

    Sample command output::

        name                : br-int
        other_config        : {disable-in-band="true", mac-table-size="2048"}
        name                : br-tun
        other_config        : {}

    Examples:
        >>> bridge_lists[0]["name"]
        'br-int'
        >>> bridge_lists[0]["other_config"]["mac-table-size"]
        '2048'
        >>> bridge_lists[0]["other_config"]["disable-in-band"]
        'true'
        >>> bridge_lists[1].get("name")
        'br-tun'
        >>> len(bridge_lists[1]["other_config"]) == 0
        True

    Attributes:
        data (list): A list containing dictionary elements where each
                     element contains the details of a bridge.

    Raises:
        SkipException: When file is empty.
    """
    def __init__(self, *args, **kwargs):
        deprecated(
            OVSvsctlListBridge,
            "Please use the :class:`insights.ovs_vsctl.OVSvsctlListBridge` instead.",
            "3.2.25"
        )
        super(OVSvsctlListBridge, self).__init__(*args, **kwargs)



"""
NeutronPlugin - file ``/etc/neutron/plugin.ini``
================================================
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.neutron_plugin_ini)
class NeutronPlugin(IniConfigFile):
    """
    Parse the ``/etc/neutron/plugin.ini`` configuration file.

    Sample configuration file::

        [ml2]
        type_drivers = local,flat,vlan,gre,vxlan
        tenant_network_types = local,flat,vlan,gre,vxlan
        mechanism_drivers = openvswitch,linuxbridge
        extension_drivers =

        [ml2_type_flat]
        flat_networks = *

        [ml2_type_vlan]
        network_vlan_ranges = physnet1:1000:2999

        [ml2_type_gre]
        tunnel_id_ranges = 20:100

        [ml2_type_vxlan]
        vni_ranges = 10:100
        vxlan_group = 224.0.0.1

        [ml2_type_geneve]
        vni_ranges =

        [securitygroup]
        enable_security_group = True

    Examples:
        >>> type(conf)
        <class 'insights.parsers.neutron_plugin.NeutronPlugin'>
        >>> 'ml2' in conf
        True
        >>> conf.has_option('ml2', 'type_drivers')
        True
        >>> conf.get("ml2_type_flat", "flat_networks")
        '*'
    """
    pass



#!/usr/bin/env python
"""
Skip Component Example
======================
This module simulates a situation where you might need different parsers
that use the same spec to parse a log file or a configuration file that
may have different formats across RHEL versions.
The idea is that, for efficiency, you only want the parser to try to parse
content that it was designed to parse.

This component can be run against the local host using the following command::

    $ insights-run -p examples.rules.skip_component

or from the examples/rules directory::

    $ ./skip_component.py
"""
from __future__ import print_function
from collections import namedtuple

from insights import get_active_lines, parser, Parser
from insights import make_fail, make_pass, rule, run
from insights.core.spec_factory import SpecSet, simple_file
from insights.combiners.redhat_release import RedHatRelease
from insights.core.plugins import component
from insights.core.dr import SkipComponent
from insights.components.rhel_version import IsRhel6, IsRhel7

# Error key used in make_fail
ERROR_KEY = "TOO_MANY_HOSTS"

# jinga2 template displayed for rule responses

CONTENT = {
    make_fail: """Too many hosts in /etc/hosts: {{num}}""",
    make_pass: """Just right"""
}


@component(RedHatRelease)
class IsRhel8(object):
    """
    This component uses ``RedhatRelease`` combiner
    to determine RHEL version. It checks if RHEL8, if
    not RHEL8 it raises ``SkipComponent``.

    Raises:
        SkipComponent: When RHEL version is not RHEL8.
    """
    def __init__(self, rhel):
        if rhel.major != 8:
            raise SkipComponent('Not RHEL8')


class Specs(SpecSet):
    """ Datasources for collection from local host """
    hosts = simple_file("/etc/hosts")


# If this runs on a server that is not Fedora or with an archive that
# is not from a Fedora server this parser will not fire since the
# Is_Fedora component raises SkipComponent if not Fedora
class HostParser(Parser):
    """
    Parses the results of the ``hosts`` Specs

    Attributes:
        hosts (list): List of the namedtuple Host
            which are the contents of the hosts file
            including ``.ip``, ``.host``, and ``.aliases``.
    """
    Host = namedtuple("Host", ["ip", "host", "aliases"])

    def parse_content(self, content):
        """
        Method to parse the contents of file ``/etc/hosts``

        This method must be implemented by each parser.

        Arguments:
            content (list): List of strings that are the contents
                of the /etc/hosts file.
        """
        self.hosts = []
        for line in get_active_lines(content):
            # remove inline comments
            line = line.partition("#")[0].strip()

            # break the line into parts
            parts = line.split()
            ip, host = parts[:2]
            aliases = parts[2:]

            self.hosts.append(HostParser.Host(ip, host, aliases))

    def __repr__(self):
        """ str: Returns string representation of the class """
        me = self.__class__.__name__
        msg = "%s([" + ", ".join([str(d) for d in self.hosts]) + "])"
        return msg % me


@parser(Specs.hosts, IsRhel8)
class ParseRhel8(HostParser):
    """
    Parser only processes content for RHEL8 Hosts, if not
    RHEL8 the parser will not fire

    Arguments:
        hp (HostParser): Parser object for the custom parser in this
            module.
    """
    pass


@parser(Specs.hosts, [IsRhel6, IsRhel7])
class ParseRhelAll(HostParser):
    """
    Parser only processes content for Rhel 6, 7 Hosts, if not
    Rhel 6, 7 the parser will not fire

    Arguments:
        hp (HostParser): Parser object for the custom parser in this
            module.
    """
    pass


# This rule will only fire for `RHEL8 Hosts' because the parser is short circuited
# for any host except `RHEL8`
@rule(ParseRhel8, content=CONTENT)
def report_rhel8(hp):
    """
    Rule reports a response if there is more than 1 host
    entry defined in the /etc/hosts file.

    Arguments:
        hp (ParserFedoraHosts): Parser object for the custom parser in this
            module. This parser will only fire if the content is from a Fedora server
    """

    if len(hp.hosts) > 1:
        return make_fail("TOO_MANY_HOSTS", num=len(hp.hosts))
    return make_pass("TOO_MANY_HOSTS", num=len(hp.hosts))


# This rule will only fire for `Rhel 6 or 7 Hosts' because the parser is short circuited
# for any host except `Rhel 6 or 7`
@rule(ParseRhelAll, content=CONTENT)
def report_rhel_others(hp):
    """
    Rule reports a response if there is more than 1 host
    entry defined in the /etc/hosts file.

    Arguments:
        hp (HostParser): Parser object for the custom parser in this
            module.
    """

    if len(hp.hosts) > 1:
        return make_fail("TOO_MANY_HOSTS", num=len(hp.hosts))
    return make_pass("TOO_MANY_HOSTS", num=len(hp.hosts))


if __name__ == "__main__":
    run(report_rhel8, print_summary=True)
    run(report_rhel_others, print_summary=True)



import doctest

from insights.parsers import nginx_conf
from insights.tests import context_wrap


NGINXCONF = """
user       root;
worker_processes  5;
error_log  logs/error.log;
pid        logs/nginx.pid;
worker_rlimit_nofile 8192;

events {
  worker_connections  4096;
}

mail {
  server_name mail.example.com;
  auth_http  localhost:9000/cgi-bin/auth;
  server {
    listen   143;
    protocol imap;
  }
}

http {
  include    conf/mime.types;
  include    /etc/nginx/proxy.conf;
  include    /etc/nginx/fastcgi.conf;
  index    index.html index.htm index.php;
  log_format  main  '$remote_addr - $remote_user [$time_local] "$request" '
                      '$status $body_bytes_sent "$http_referer" '
                      '"$http_user_agent" "$http_x_forwarded_for"';

  default_type application/octet-stream;
  access_log   logs/access.log  main;
  sendfile     on;
  tcp_nopush   on;
  server_names_hash_bucket_size 128;

  server { # php/fastcgi
    listen       80;
    server_name  domain1.com www.domain1.com;
    access_log   logs/domain1.access.log  main;
    root         html;

    location ~ \.php$ {
      fastcgi_pass   127.0.0.1:1025;
    }
  }

  server { # simple reverse-proxy
    listen       80;
    server_name  domain2.com www.domain2.com;
    access_log   logs/domain2.access.log  main;

    location ~ ^/(images|javascript|js|css|flash|media|static)/  {
      root    /var/www/virtual/big.server.com/htdocs;
      expires 30d;
    }

    location / {
      proxy_pass   http://127.0.0.1:8080;
    }
  }

  map $http_upgrade $connection_upgrade {
    default upgrade;
    '' close;
  }

  upstream websocket {
    server 10.66.208.205:8010;
  }

  upstream big_server_com {
    server 127.0.0.3:8000 weight=5;
    server 127.0.0.3:8001 weight=5;
    server 192.168.0.1:8000;
    server 192.168.0.1:8001;
  }

  server { # simple load balancing
    listen          80;
    server_name     big.server.com;
    access_log      logs/big.server.access.log main;

    location / {
      proxy_pass      http://big_server_com;
      location /inner/ {
         proxy_pass http://u2;
         limit_except GET {
             allow 192.168.2.0/32;
         }
      }
    }
  }
}
""".strip()


def test_nginxconfiguration():
    nginxconf = nginx_conf.NginxConf(context_wrap(NGINXCONF))
    assert nginxconf['user'] == 'root'
    assert nginxconf['events']['worker_connections'] == '4096'
    assert nginxconf['mail']['server'][0]['listen'] == '143'
    assert nginxconf['http']['access_log'] == 'logs/access.log  main'
    assert nginxconf['http']['server'][0]['location'][0]['fastcgi_pass'] == '127.0.0.1:1025'
    assert nginxconf['http']['server'][1]['location'][1]['name'] == '/'
    assert nginxconf['http']['upstream'][1]['name'] == 'big_server_com'
    assert nginxconf["http"]["include"][0] == 'conf/mime.types'
    assert nginxconf['http']['upstream'][1]['server'][0] == '127.0.0.3:8000 weight=5'
    assert nginxconf['http']['log_format'] == """main  '$remote_addr - $remote_user [$time_local] "$request" '
'$status $body_bytes_sent "$http_referer" '
'"$http_user_agent" "$http_x_forwarded_for"'"""
    assert nginxconf['http']['server'][2]['location'][0]['location'][0]['limit_except']['allow'] == '192.168.2.0/32'


def test_nginxconfpeg():
    nginxconf = nginx_conf.NginxConfPEG(context_wrap(NGINXCONF))
    assert nginxconf['user'][-1].value == 'root'
    assert nginxconf['events'][-1]['worker_connections'][-1].value == 4096
    assert nginxconf['mail'][-1]['server'][0]['listen'][-1].value == 143
    assert nginxconf['http'][-1]['access_log'][-1].value == 'logs/access.log main'
    assert nginxconf['http'][-1]['server'][0]['location'][0]['fastcgi_pass'][-1].value == '127.0.0.1:1025'
    assert nginxconf['http'][-1]['server'][1]['location'][-1].value == '/'
    assert nginxconf['http'][-1]['upstream'][1].value == 'big_server_com'
    assert nginxconf["http"][-1]["include"][0].value == 'conf/mime.types'
    assert nginxconf['http'][-1]['upstream'][1]['server'][0].value == '127.0.0.3:8000 weight=5'
    assert nginxconf['http'][-1]['log_format'][
               -1].value == 'main $remote_addr - $remote_user [$time_local] "$request"  $status $body_bytes_sent "$http_referer"  "$http_user_agent" "$http_x_forwarded_for"'
    assert nginxconf['http'][-1]['server'][2]['location'][0]['location'][0]['limit_except'][-1]['allow'][
               -1].value == '192.168.2.0/32'
    assert nginxconf['http']['server']['location']['location']['limit_except']['allow'][-1].value == '192.168.2.0/32'
    assert nginxconf['http']['server'][0]['location'][-1].value == r'~ \.php$'
    assert nginxconf['http']['server'][1]['location'][0].value == '~ ^/(images|javascript|js|css|flash|media|static)/'
    assert nginxconf['http']['server'][1]['location'][-1].value == '/'
    assert nginxconf['http']['server'][-1] == nginxconf['http']['server'][2]


def test_nginxconfpeg_container():
    nginxconf = nginx_conf.ContainerNginxConfPEG(
        context_wrap(
            NGINXCONF,
            engine='podman',
            image='quay.io/rhel8',
            container_id='xxxx'
        )
    )
    assert nginxconf['user'][-1].value == 'root'
    assert nginxconf['events'][-1]['worker_connections'][-1].value == 4096
    assert nginxconf['mail'][-1]['server'][0]['listen'][-1].value == 143
    assert nginxconf['http'][-1]['access_log'][-1].value == 'logs/access.log main'
    assert nginxconf['http'][-1]['server'][0]['location'][0]['fastcgi_pass'][-1].value == '127.0.0.1:1025'
    assert nginxconf['http'][-1]['server'][1]['location'][-1].value == '/'
    assert nginxconf['http'][-1]['upstream'][1].value == 'big_server_com'
    assert nginxconf["http"][-1]["include"][0].value == 'conf/mime.types'
    assert nginxconf['http'][-1]['upstream'][1]['server'][0].value == '127.0.0.3:8000 weight=5'
    assert nginxconf['http'][-1]['log_format'][
               -1].value == 'main $remote_addr - $remote_user [$time_local] "$request"  $status $body_bytes_sent "$http_referer"  "$http_user_agent" "$http_x_forwarded_for"'
    assert nginxconf['http'][-1]['server'][2]['location'][0]['location'][0]['limit_except'][-1]['allow'][
               -1].value == '192.168.2.0/32'
    assert nginxconf['http']['server']['location']['location']['limit_except']['allow'][-1].value == '192.168.2.0/32'
    assert nginxconf['http']['server'][0]['location'][-1].value == r'~ \.php$'
    assert nginxconf['http']['server'][1]['location'][0].value == '~ ^/(images|javascript|js|css|flash|media|static)/'
    assert nginxconf['http']['server'][1]['location'][-1].value == '/'
    assert nginxconf['http']['server'][-1] == nginxconf['http']['server'][2]
    assert nginxconf.engine == 'podman'
    assert nginxconf.image == 'quay.io/rhel8'
    assert nginxconf.container_id == 'xxxx'


def test_doc():
    env = {
        'nginxconf': nginx_conf.NginxConf(context_wrap(NGINXCONF)),
        'nginxconfpeg': nginx_conf.NginxConfPEG(context_wrap(NGINXCONF)),
    }
    failed, total = doctest.testmod(nginx_conf, globs=env)
    assert failed == 0



"""
AuditctlStatus - Report auditd status
=====================================
"""

from .. import parser, CommandParser, LegacyItemAccess
from ..parsers import ParseException
from ..specs import Specs
from insights.util import deprecated


@parser(Specs.auditctl_status)
class AuditctlStatus(LegacyItemAccess, CommandParser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.auditctl.AuditdStatus` instead.

    Module for parsing the output of the ``auditctl -s`` command.

    Typical output on RHEL6 looks like::

        AUDIT_STATUS: enabled=1 flag=1 pid=1483 rate_limit=0 backlog_limit=8192 lost=3 backlog=0

    , while on RHEL7 the output changes to::

        enabled 1
        failure 1
        pid 947
        rate_limit 0
        backlog_limit 320
        lost 0
        backlog 0
        loginuid_immutable 0 unlocked

    Example:
        >>> type(auds)
        <class 'insights.parsers.auditctl_status.AuditctlStatus'>
        >>> "enabled" in auds
        True
        >>> auds['enabled']
        1
    """
    def __init__(self, context):
        deprecated(
            AuditctlStatus,
            "Please use the :class:`insights.parsers.auditctl.AuditdStatus` instead.",
            "3.1.25"
        )
        super(AuditctlStatus, self).__init__(context)

    def parse_content(self, content):
        if not content:
            raise ParseException("Input content is empty.")
        self.data = {}
        if len(content) > 1:
            for line in content:
                k, v = line.split(None, 1)
                # Mind the 'loginuid_immutable' on RHEL7
                if k.strip() == "loginuid_immutable":
                    self.data[k.strip()] = v.strip()
                else:
                    try:
                        self.data[k.strip()] = int(v.strip())
                    except ValueError:
                        continue
        if len(content) == 1:
            line = list(content)[0].strip()
            if line.startswith("AUDIT_STATUS:"):
                for item in line.split(None)[1:]:
                    try:
                        k, v = item.split('=')
                        self.data[k.strip()] = int(v.strip())
                    except ValueError:
                        continue



"""
Satellite PostgreSQL database queries
=====================================

This module contains the following parsers:

SatelliteAdminSettings - command ``psql -d foreman -c 'select name, value, "default" from settings where name in (\'destroy_vm_on_host_delete\', \'unregister_delete_host\') --csv'``
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
SatelliteComputeResources - command ``psql -d foreman -c 'select name, type from compute_resources' --csv``
-----------------------------------------------------------------------------------------------------------
SatelliteCoreTaskReservedResourceCount - command ``psql -d pulpcore -c 'select count(*) from core_taskreservedresource' --csv``
-------------------------------------------------------------------------------------------------------------------------------
SatelliteKatellloReposWithMultipleRef - command ``psql -d foreman -c "select repository_href, count(*) from katello_repository_references group by repository_href having count(*) > 1;" --csv``
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
SatelliteLogsTableSize - command ``psql -d foreman -c "select pg_total_relation_size('logs') as logs_size" --csv``
------------------------------------------------------------------------------------------------------------------
SatelliteProvisionParamSettings - command ``psql -d foreman -c "select name, value from parameters where name='package_upgrade' and reference_id in (select id from operatingsystems where name='RedHat' and major='9')" --csv``
--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
SatelliteQualifiedCapsules - command ``psql -d foreman -c "select name from smart_proxies where download_policy = 'background'" --csv``
---------------------------------------------------------------------------------------------------------------------------------------
SatelliteQualifiedKatelloRepos - command ``psql -d foreman -c "select id, name, url, download_policy from katello_root_repositories where download_policy = 'background' or url is NULL" --csv``
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
SatelliteSCAStatus - command ``psql -d candlepin -c "select displayname, content_access_mode from cp_owner" --csv``
-------------------------------------------------------------------------------------------------------------------
"""

import os
import yaml
from csv import DictReader

from insights import parser, CommandParser
from insights.specs import Specs
from insights.parsers import SkipException, ParseException
from insights.parsers import keyword_search, calc_offset
from insights.util import deprecated


class SatellitePostgreSQLQuery(CommandParser, list):
    """
    Parent class of satellite postgresql table queries. It can not be used
    directly, A child class with overriding columns attribute is required.
    It saves the rows data into a list. Each row is saved into a dict.
    The key is the column name, the value is the value of the column.

    Resultant data structure::

        [
            {
                'name': 'abc',
                'url': '',
                'value': 'test'
            },
            {
                'name': 'def',
                'url': 'http://xx.com',
                'value': ''
            }
        ]

    Sample Output::

        name,url,value
        abc,,test
        def,http://xx.com,

    Raises:
        SkipException: when there isn't data in the table
        ParseException: when the output isn't in good csv format or the yaml values aren't in good yaml format
        NotImplementedError: when the subclass doesn't override the columns attribute.
    """

    # child class should override the columns attribute with its own column names
    columns = []
    # child class should define the columns_in_yaml when some columns value are in yaml format,
    # which should be transfered to python object
    columns_in_yaml = []

    def _parse_yaml(self, value):
        if value:
            try:
                return yaml.safe_load(value)
            except Exception:
                raise ParseException("Bad format value: %s" % value)
        return value

    def parse_content(self, content):
        if not self.columns:
            raise NotImplementedError("Please override the columns attribute.")
        start_index = calc_offset(content, self.columns, require_all=True)
        if 'Last login:' in content[-1]:
            valid_lines = content[start_index:-1]
        else:
            valid_lines = content[start_index:]
        reader = DictReader(os.linesep.join(valid_lines).splitlines(True))
        for row in reader:
            for name in self.columns_in_yaml:
                row[name] = self._parse_yaml(row[name])
            self.append(row)
        if not self:
            raise SkipException("There is no data in the table.")

    def search(self, **kwargs):
        """
        Get the rows by searching the table with kwargs.
        This uses the :py:func:`insights.parsers.keyword_search` function for
        searching; see its documentation for usage details. If no search
        parameters are given, no rows are returned.

        It simplify the value of the column according to actual usage.

        Returns:
            list: A list of dictionaries of rows that match the given
            search criteria.
        """

        return keyword_search(self, **kwargs)


@parser(Specs.satellite_settings)
class SatelliteAdminSettings(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c '"select name, value, "default" from settings where name in ('destroy_vm_on_host_delete', 'unregister_delete_host') --csv"``.

    Sample output::

        name,value,default
        unregister_delete_host,"--- true
        ...","--- false
        ..."
        destroy_vm_on_host_delete,,"--- true
        ..."

    Examples:
        >>> type(table)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteAdminSettings'>
        >>> table.get_setting('unregister_delete_host')
        True
        >>> table.get_setting('destroy_vm_on_host_delete')
        True
    """
    columns = ['name', 'value', 'default']
    columns_in_yaml = ['value', 'default']

    def get_setting(self, setting_name):
        """
        Get the actual value of setting_name.
        If the value column isn't empty, the value of the setting_name is the
        value column, or else it's the default column.

        Args:
            setting_name (str): the value of name column which is searched in the table.

        Returns:
            It depends on the setting, maybe boolean, string, int or a list.
            None if the setting_name doesn't exist in the table.
        """
        rows = self.search(name=setting_name)
        if rows:
            value = rows[0].get('value')
            return rows[0].get('default') if value == '' else value


@parser(Specs.satellite_compute_resources)
class SatelliteComputeResources(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c 'select name, type from compute_resources' --csv``.

    Sample output::

        name,type
        test_compute_resource1,Foreman::Model::Libvirt
        test_compute_resource2,Foreman::Model::RHV

    Examples:
        >>> type(resources_table)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteComputeResources'>
        >>> rows=resources_table.search(type='Foreman::Model::Libvirt')
        >>> len(rows)
        1
        >>> rows[0]['name']
        'test_compute_resource1'
    """
    columns = ['name', 'type']


@parser(Specs.satellite_core_taskreservedresource_count)
class SatelliteCoreTaskReservedResourceCount(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d pulpcore -c 'select count(*) from core_taskreservedresource' --csv``.

    Sample output::

        count
        0

    Examples:
        >>> type(tasks)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteCoreTaskReservedResourceCount'>
        >>> tasks[0]['count']
        '0'
    """
    columns = ['count']


@parser(Specs.satellite_katello_empty_url_repositories)
class SatelliteKatelloEmptyURLRepositories(SatellitePostgreSQLQuery):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.satellite_postgresql_query.SatelliteQualifiedKatelloRepos` instead.

    Parse the output of the command ``psql -d foreman -c 'select id, name from katello_root_repositories where url is NULL;' --csv``.

    Sample output::

        id,name
        54,testa
        55,testb

    Examples:
        >>> type(katello_root_repositories)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteKatelloEmptyURLRepositories'>
        >>> len(katello_root_repositories)
        2
        >>> katello_root_repositories[0]['name']
        'testa'
    """
    columns = ['id', 'name']

    def __init__(self, *args, **kwargs):
        deprecated(
            SatelliteKatelloEmptyURLRepositories,
            "Please use the SatelliteQualifiedKatelloRepos parser in the current module.",
            "3.1.25"
        )
        super(SatelliteKatelloEmptyURLRepositories, self).__init__(*args, **kwargs)


@parser(Specs.satellite_logs_table_size)
class SatelliteLogsTableSize(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c "select pg_total_relation_size('logs') as logs_size" --csv``.

    Sample output::

        logs_size
        565248

    Examples:
        >>> type(logs_table)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteLogsTableSize'>
        >>> logs_table[0]['logs_size']
        565248

    Raises:
        ParseException: when the size is not in integer type
    """
    columns = ['logs_size']

    def parse_content(self, content):
        super(SatelliteLogsTableSize, self).parse_content(content)
        for row in self:
            if not row['logs_size'].isdigit():
                raise ParseException('Not expected logs size %s.' % row['logs_size'])
            row['logs_size'] = int(row['logs_size'])


@parser(Specs.satellite_provision_param_settings)
class SatelliteProvisionParamSettings(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c "select name, value from parameters where name='package_upgrade' and reference_id in (select id from operatingsystems where name='RedHat' and major='9')" --csv``.

    Sample output::

        name,value
        package_upgrade,"--- false
        ...
        "

    Examples:
        >>> type(param_settings)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteProvisionParamSettings'>
        >>> len(param_settings)
        1
        >>> param_settings[0]['value']
        False
    """
    columns = ['name', 'value']
    columns_in_yaml = ['value']


@parser(Specs.satellite_katello_repos_with_muliple_ref)
class SatelliteKatellloReposWithMultipleRef(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c "select repository_href, count(*) from katello_repository_references group by repository_href having count(*) > 1;" --csv``.

    Sample output::

        repository_href,count
        /pulp/api/v3/repositories/rpm/rpm/64e1ddf8-025e-45f2-b2f0-04b874674671/,3
        /pulp/api/v3/repositories/rpm/rpm/sfwrsrw45sfse-45f2-b2f0-04b874675688/,2

    Examples:
        >>> type(multi_ref_katello_repos)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteKatellloReposWithMultipleRef'>
        >>> len(multi_ref_katello_repos)
        2
        >>> multi_ref_katello_repos[0]['repository_href']
        '/pulp/api/v3/repositories/rpm/rpm/64e1ddf8-025e-45f2-b2f0-04b874674671/'
    """
    columns = ['repository_href', 'count']


@parser(Specs.satellite_qualified_katello_repos)
class SatelliteQualifiedKatelloRepos(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c "select id, name, url, download_policy from katello_root_repositories where download_policy = 'background' or url is NULL" --csv``.

    Sample output::

        id,name,url,download_policy
        2,Red Hat Satellite Tools 6.8 for RHEL 7 Server RPMs x86_64,,on_demand
        3,Red Hat Enterprise Linux 8 for x86_64 - AppStream RPMs 8,https://cdn.redhat.com/content/dist/rhel8/8/x86_64/appstream/os,background
        4,Red Hat Enterprise Linux 7 Server RPMs x86_64 7Server,https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/os,background

    Examples:
        >>> type(repos)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteQualifiedKatelloRepos'>
        >>> len(repos)
        3
        >>> repos[0]['name']
        'Red Hat Satellite Tools 6.8 for RHEL 7 Server RPMs x86_64'
    """
    columns = ['id', 'name', 'url', 'download_policy']


@parser(Specs.satellite_qualified_capsules)
class SatelliteQualifiedCapsules(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d foreman -c "select name from smart_proxies where download_policy = 'background'" --csv``.

    Sample output::

        name
        capsule1.test.com
        capsule2.test.com

    Examples:
        >>> type(capsules)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteQualifiedCapsules'>
        >>> len(capsules)
        2
        >>> capsules[0]['name']
        'capsule1.test.com'
    """
    columns = ['name']


@parser(Specs.satellite_sca_status)
class SatelliteSCAStatus(SatellitePostgreSQLQuery):
    """
    Parse the output of the command ``psql -d candlepin -c "select displayname, content_access_mode from cp_owner" --csv``.

    Sample output::

        displayname,content_access_mode
        Default Organization,entitlement
        Orgq,org_environment

    Examples:
        >>> type(sat_sca_info)
        <class 'insights.parsers.satellite_postgresql_query.SatelliteSCAStatus'>
        >>> sat_sca_info.sca_enabled
        True
    """

    columns = ['displayname', 'content_access_mode']

    @property
    def sca_enabled(self):
        """
        If the value of content_access_mode is "org_environment", it means the SCA is enabled for this organization.
        Return True if any organization has SCA enabled on the satellite else False
        """
        return bool(len(self.search(content_access_mode='org_environment')))



import doctest
import pytest

from insights.parsers import satellite_missed_queues
from insights.tests import context_wrap
from insights import SkipComponent


SATELLITE_MISSED_QUEUES_OUTPUT1 = """
pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9:2018-01-16 00:06:13
pulp.agent.fac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:16
0
""".strip()

SATELLITE_MISSED_QUEUES_OUTPUT2 = """
pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9:2018-01-16 00:06:13
pulp.agent.iac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:16
pulp.agent.aac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:17
pulp.agent.bac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:18
pulp.agent.cac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:19
pulp.agent.dac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:20
pulp.agent.eac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:21
pulp.agent.fac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:22
pulp.agent.gac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:23
pulp.agent.hac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:24
1
""".strip()

SATELLITE_MISSED_QUEUES_BAD_OUTPUT_1 = """
pulp.agent.09008eec-aba6-4174-aa9f-e930004ce5c9:2018-01-16 00:06:13
""".strip()

SATELLITE_MISSED_QUEUES_BAD_OUTPUT_2 = """
abc
def
""".strip()

SATELLITE_MISSED_QUEUES_BAD_OUTPUT_3 = """
pulp.agent.eac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:21
pulp.agent.fac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:22
pulp.agent.gac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:23
pulp.agent.hac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024:2018-01-16 00:06:24
""".strip()


def test_satellite_queue():
    queues = satellite_missed_queues.SatelliteMissedQueues(context_wrap(SATELLITE_MISSED_QUEUES_OUTPUT2))
    assert queues.truncated
    assert len(queues.missed_queues) == 10
    assert 'pulp.agent.hac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024' in queues.missed_queues
    assert queues.missed_queues['pulp.agent.hac7ebbc-ee4f-44b4-9fe0-3f4e42c7f024'] == '2018-01-16 00:06:24'


def test_doc_examples():
    env = {
        'satellite_queues': satellite_missed_queues.SatelliteMissedQueues(context_wrap(SATELLITE_MISSED_QUEUES_OUTPUT1)),
    }
    failed, total = doctest.testmod(satellite_missed_queues, globs=env)
    assert failed == 0


def test_exception():
    with pytest.raises(SkipComponent):
        satellite_missed_queues.SatelliteMissedQueues(context_wrap(SATELLITE_MISSED_QUEUES_BAD_OUTPUT_1))
    with pytest.raises(SkipComponent):
        satellite_missed_queues.SatelliteMissedQueues(context_wrap(SATELLITE_MISSED_QUEUES_BAD_OUTPUT_2))
    with pytest.raises(SkipComponent):
        satellite_missed_queues.SatelliteMissedQueues(context_wrap(SATELLITE_MISSED_QUEUES_BAD_OUTPUT_3))



import shutil
import tempfile
from contextlib import closing

import os
import pytest

from insights.util import fs


def test_remove_exists():
    with closing(tempfile.NamedTemporaryFile(delete=False)) as f:
        full_path = f.name
        f.write(b'test')

    assert os.path.exists(full_path)
    fs.remove(full_path)
    assert not os.path.exists(full_path)


def test_remove_not_exists():
    full_path = os.path.expandvars('$PWD/not/a/path')
    assert not os.path.exists(full_path)
    fs.remove(full_path)
    assert not os.path.exists(full_path)


def test_not_exists_ensure_path():
    base_path = os.path.expandvars('$PWD/a')
    path = os.path.expandvars('$PWD/a/b/c')

    assert not os.path.exists(base_path)

    fs.ensure_path(path)
    assert os.path.exists(path)

    shutil.rmtree(base_path)
    assert not os.path.exists(base_path)


def test_exists_ensure_path_exists():
    path = os.path.expandvars('$PWD')
    assert os.path.exists(path)
    fs.ensure_path(path)
    assert os.path.exists(path)


@pytest.fixture(params=[('644', '644'), ('775', '755'), ('777', '755')])
def mode(request):
    yield request.param


@pytest.fixture
def mode_dir(mode):
    path = os.path.join('/tmp/insights-test-{mode}'.format(mode=mode[0]))
    yield path
    fs.remove(path)


def test_ensure_path_mode(mode, mode_dir):
    fs.ensure_path(mode_dir, int(mode[0], 8))
    assert oct(os.stat(mode_dir).st_mode)[-3:] == mode[1]


def test_ensure_path_mode_default():
    path = os.path.join('/tmp/insights-test-default')
    fs.ensure_path(path)
    assert oct(os.stat(path).st_mode)[-3:] == '755'
    fs.remove(path)


def test_touch_times():
    path = '/tmp/insights-test-touch-times'
    fs.touch(path, (1259798405, 1259798400))
    assert os.stat(path).st_atime == 1259798405
    assert os.stat(path).st_mtime == 1259798400
    fs.remove(path)



from insights.parsers.limits_conf import LimitsConf
from insights.combiners.limits_conf import AllLimitsConf
from insights.tests import context_wrap

LIMITS_CONF = """
#oracle soft nproc 2047
#oracle hard nproc 16384
oracle soft nofile 1024
oracle hard nofile 65536
oracle soft stack 10240
oracle hard stack 3276
root       soft    nproc     unlimited
""".strip()

LIMITS_D_CONF = """
@jackuser - rtprio 70
@jackuser - memlock 4194304
""".strip()

DUP_LIMITS_D_CONF = """
oracle hard stack 1926
"""

LIMITS_CONF_PATH = "/etc/security/limits.conf"
LIMITS_D_PATH = "/etc/security/limits.d/95-jack.conf"
DUP_LIMITS_D_CONF_PATH = "/etc/security/limits.d/oracle.conf"


def test_all_limits_conf():
    data1 = LimitsConf(context_wrap(LIMITS_CONF, path=LIMITS_CONF_PATH))
    data2 = LimitsConf(context_wrap(LIMITS_D_CONF, path=LIMITS_D_PATH))
    all_data = AllLimitsConf([data1, data2])

    assert len(all_data.rules) == 7
    assert all_data.rules[0] == {'domain': 'oracle', 'type': 'soft', 'item': 'nofile', 'value': 1024, 'file': LIMITS_CONF_PATH}
    assert all_data.rules[1] == {'domain': 'oracle', 'type': 'hard', 'item': 'nofile', 'value': 65536, 'file': LIMITS_CONF_PATH}
    assert all_data.rules[2] == {'domain': 'oracle', 'type': 'soft', 'item': 'stack', 'value': 10240, 'file': LIMITS_CONF_PATH}
    assert all_data.rules[3] == {'domain': 'oracle', 'type': 'hard', 'item': 'stack', 'value': 3276, 'file': LIMITS_CONF_PATH}
    assert all_data.rules[4] == {'domain': 'root', 'type': 'soft', 'item': 'nproc', 'value': -1, 'file': LIMITS_CONF_PATH}
    assert all_data.rules[5] == {'domain': '@jackuser', 'type': '-', 'item': 'rtprio', 'value': 70, 'file': LIMITS_D_PATH}
    assert all_data.rules[6] == {'domain': '@jackuser', 'type': '-', 'item': 'memlock', 'value': 4194304, 'file': LIMITS_D_PATH}

    assert all_data.domains == set(['oracle', 'root', '@jackuser'])

    assert all_data.find_all(domain='root') == [all_data.rules[4]]

    data3 = LimitsConf(context_wrap(DUP_LIMITS_D_CONF, path=DUP_LIMITS_D_CONF_PATH))
    all_data = AllLimitsConf([data1, data3])
    # Check de-duplication of rules
    assert len(all_data.rules) == 5  # No extra rule
    assert all_data.rules[3] == {'domain': 'oracle', 'type': 'hard', 'item': 'stack', 'value': 1926, 'file': DUP_LIMITS_D_CONF_PATH}



from insights.parsr import Colon, QuotedString


def test_keepleft():
    key = QuotedString << Colon
    assert key('"key":') == "key"

    key = Colon << QuotedString
    assert key(':"key"') == ":"


def test_keepright():
    key = QuotedString >> Colon
    assert key('"key":') == ":"

    key = Colon >> QuotedString
    assert key(':"key"') == "key"


def test_middle():
    key = Colon >> QuotedString << Colon
    assert key(':"key":') == "key"
